<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Comparimager for Elementor
 * Customizable image compare for Elementor editor
 * Exclusively on https://1.envato.market/comparimager-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Css_Filter;
use Elementor\Group_Control_Typography;
use Elementor\Icons_Manager;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Utils;
use Elementor\Group_Control_Image_Size;
use Elementor\Plugin;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Comparimager - Custom Elementor Widget.
 **/
class comparimager_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style(
                'mdp-selection-elementor-admin',
                    UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
                        [],
                        UnityPlugin::get_version()
        );
        wp_register_style(
                'mdp-comparimager-elementor',
                    UnityPlugin::get_url() . 'css/comparimager-elementor' . UnityPlugin::get_suffix() . '.css',
                        [],
                        UnityPlugin::get_version()
        );
	    wp_register_script(
                'mdp-comparimager-elementor',
                    UnityPlugin::get_url() . 'js/comparimager-elementor' . UnityPlugin::get_suffix() . '.js',
                        [ 'jquery' ],
                        UnityPlugin::get_version(),
               true
        );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-comparimager-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Image Compare', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-comparimager-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Image Compare' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-comparimager-elementor', 'mdp-selection-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'mdp-comparimager-elementor' ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> General Content Section. */
        $this->section_content_general();

        /** Content -> Image Content Section. */
        $this->section_content_image();

        /** Content -> Compare Line Content Section. */
        $this->section_content_compare_line();

    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Style Before Image. */
        $this->section_style_before_image();

        /** Style -> Section Style After Image. */
        $this->section_style_after_image();

        /** Style -> Section Style Label Before. */
        $this->section_style_label_before();

        /** Style -> Section Style Label After. */
        $this->section_style_label_after();

        /** Style -> Section Style Overlay. */
        $this->section_style_overlay();

        /** Style -> Section Style Compare Line. */
        $this->section_style_compare_line();

        /** Style -> Section Style Compare Line Button. */
        $this->section_style_compare_line_button();

    }

    /**
     * Add widget controls: Content -> General Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_general() {

        $this->start_controls_section( 'section_content_general', [
            'label' => esc_html__( 'General', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'default_offset',
            [
                'label' => esc_html__( 'Default offset', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1
                    ]
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-comparimager-elementor-position-vertical .mdp-comparimager-elementor-compare-handle' => 'left: {{SIZE}}{{UNIT}}; top: 0;',
                    '{{WRAPPER}} .mdp-comparimager-elementor-position-horizontal .mdp-comparimager-elementor-compare-handle' => 'top: {{SIZE}}{{UNIT}}; left: 0;'
                ]
            ]
        );

        $this->add_responsive_control(
            'slider_height',
            [
                'label' => esc_html__( 'Slider height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'size' => '390',
                    'unit' => 'px'
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-comparimager-elementor-comparison-slider' => 'height: {{SIZE}}{{UNIT}};',
                ]
            ]
        );

	    $this->add_control(
		    'show_overlay',
		    [
			    'label' => esc_html__( 'Overlay', 'selection' ),
			    'type' => Controls_Manager::SWITCHER,
			    'return_value' => 'yes',
			    'default' => 'yes',
		    ]
	    );

        $this->add_control(
            'show_overlay_on',
            [
                'label'     => esc_html__( 'Show overlay on', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'always'    => esc_html__( 'Always', 'selection' ),
                    'hover'    => esc_html__( 'Hover', 'selection' ),
                ],
                'default'   => 'hover',
                'condition' => [
                    'show_overlay' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'move_handle',
            [
                'label'     => esc_html__( 'Move handle', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'click'    => esc_html__( 'Click', 'selection' ),
                    'hover'    => esc_html__( 'Hover', 'selection' ),
                    'swipe'    => esc_html__( 'Swipe', 'selection' ),
                ],
                'default'   => 'click',
            ]
        );

        $this->add_control(
            'line_position',
            [
                'label'     => esc_html__( 'Line position', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'horizontal'    => esc_html__( 'Horizontal', 'selection' ),
                    'vertical'    => esc_html__( 'Vertical', 'selection' ),
                ],
                'default'   => 'vertical',
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Image Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_image() {

        $this->start_controls_section( 'section_content_image', [
            'label' => esc_html__( 'Image', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $this->start_controls_tabs( 'image_tabs' );

        $this->start_controls_tab(
            'image_before_tab',
            ['label' => esc_html__( 'Before', 'selection' )]
        );

        $this->add_control(
            'before_image_label',
            [
                'label' => esc_html__( 'Label before', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Before', 'selection' ),
                'placeholder' => esc_html__( 'Type your before label here', 'selection' ),
            ]
        );

        $this->add_control(
            'before_image',
            [
                'label' => esc_html__( 'Before image', 'selection' ),
                'type' => Controls_Manager::MEDIA,
                'default' => [
                    'url' => Utils::get_placeholder_image_src(),
                ],
            ]
        );

        $this->end_controls_tab();

        $this->start_controls_tab(
            'image_after_tab',
            ['label' => esc_html__( 'After', 'selection' )]
        );

        $this->add_control(
            'after_image_label',
            [
                'label' => esc_html__( 'Label after', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'After', 'selection' ),
                'placeholder' => esc_html__( 'Type your after label here', 'selection' ),
            ]
        );

        $this->add_control(
            'after_image',
            [
                'label' => esc_html__( 'After image', 'selection' ),
                'type' => Controls_Manager::MEDIA,
                'default' => [
                    'url' => Utils::get_placeholder_image_src(),
                ],
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();

        $this->add_group_control(
            Group_Control_Image_Size::get_type(),
            [
                'name' => 'images_size',
                'exclude' => [],
                'include' => [],
                'default' => 'large',
            ]
        );



        $this->end_controls_section();

    }


    /**
     * Add widget controls: Content -> Compare Line Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_compare_line() {

        $this->start_controls_section( 'section_content_compare_line', [
            'label' => esc_html__( 'Compare line', 'selection' ),
            'tab' => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'vertical_icon_previous',
            [
                'label' => esc_html__( 'Icon previous', 'selection' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-chevron-left',
                    'library' => 'solid',
                ],
                'condition' => [
                    'line_position' => 'vertical'
                ]
            ]
        );

        $this->add_control(
            'vertical_icon_next',
            [
                'label' => esc_html__( 'Icon next', 'selection' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-chevron-right',
                    'library' => 'solid',
                ],
                'condition' => [
                    'line_position' => 'vertical'
                ]
            ]
        );

        $this->add_control(
            'horizontal_icon_previous',
            [
                'label' => esc_html__( 'Icon previous', 'selection' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-chevron-up',
                    'library' => 'solid',
                ],
                'condition' => [
                    'line_position' => 'horizontal'
                ]
            ]
        );

        $this->add_control(
            'horizontal_icon_next',
            [
                'label' => esc_html__( 'Icon next', 'selection' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-chevron-down',
                    'library' => 'solid',
                ],
                'condition' => [
                    'line_position' => 'horizontal'
                ]
            ]
        );


        $this->end_controls_section();

    }

    /**
     * Method for generating margin padding controls.
     *
     * @param $section_id
     * @param $html_class
     * @param array $default_padding
     * @param array $default_margin
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_margin_padding_controls( $section_id, $html_class, $default_padding = [], $default_margin = [] ) {


        $this->add_responsive_control(
            $section_id.'_margin',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'default' => $default_margin,
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'default' => $default_padding,
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );
    }

    /**
     * Function for generating typography and tabs controls.
     *
     * @param $section_id
     * @param $opts
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_typography_tabs_controls( $section_id, $opts = [] ) {
        $style_opts = [
            'html_class' => array_key_exists( 'html_class', $opts ) ?
                $opts['html_class'] : '',
            'active_class' => array_key_exists( 'active_class', $opts ) ?
                $opts['active_class'] : '',
            'include_color' => array_key_exists( 'include_color', $opts ) ?
                $opts['include_color'] : true,
            'include_bg' => array_key_exists( 'include_bg', $opts ) ?
                $opts['include_color'] : true,
            'include_typography' => array_key_exists( 'include_typography', $opts ) ?
                $opts['include_typography'] : true,
            'include_transition' => array_key_exists( 'include_transition', $opts ) ?
                $opts['include_transition'] : true,
            'additional_color' => array_key_exists( 'additional_color', $opts ) ?
                $opts['additional_color'] : false,
            'additional_color_hover' => array_key_exists( 'additional_color_hover', $opts ) ?
                $opts['additional_color_hover'] : false,
            'include_active_tab' => array_key_exists( 'include_active_tab', $opts ) ?
                $opts['include_active_tab'] : false,
            'include_css_filters' => array_key_exists( 'include_css_filters', $opts ) ?
                $opts['include_css_filters'] : false,
            'active_tab_name' => array_key_exists( 'active_tab_name', $opts ) ?
                $opts['active_tab_name'] : 'ACTIVE',
            'color_prefix' => array_key_exists( 'color_prefix', $opts ) ?
                $opts['color_prefix'] : '',
            'color_class' => array_key_exists( 'color_class', $opts ) ?
                $opts['color_class'] : '',
            'color_hover_class' => array_key_exists( 'color_hover_class', $opts ) ?
                $opts['color_hover_class'] : '',
            'color_active_class' => array_key_exists( 'color_active_class', $opts ) ?
                $opts['color_active_class'] : '',
            'color_hover_selector' => array_key_exists( 'color_hover_selector', $opts ) ?
                $opts['color_hover_selector'] : '',
            'additional_color_name' => array_key_exists( 'additional_color_name', $opts ) ?
                $opts['additional_color_name'] : '',
            'additional_color_class' => array_key_exists( 'additional_color_class', $opts ) ?
                $opts['additional_color_class'] : '',
            'additional_color_hover_selectors' => array_key_exists( 'additional_color_hover_selectors', $opts ) ?
                $opts['additional_color_hover_selectors'] : '',
            'additional_color_active_class' => array_key_exists( 'additional_color_active_class', $opts ) ?
                $opts['additional_color_active_class'] : '',
            'additional_transition_selector' => array_key_exists( 'additional_transition_selector', $opts ) ?
                $opts['additional_transition_selector'] : '',
            'typography_class' => array_key_exists( 'typography_class', $opts ) ?
                $opts['typography_class'] : '',
            'additional_border_radius_selector' => array_key_exists( 'additional_border_radius_selector', $opts ) ?
                $opts['additional_border_radius_selector'] : '',
            'additional_border_radius_hover_selector' => array_key_exists( 'additional_border_radius_hover_selector', $opts ) ?
                $opts['additional_border_radius_hover_selector'] : '',
            'color_scheme_default' => array_key_exists( 'color_scheme_default', $opts ) ?
                $opts['color_scheme_default'] : Color::COLOR_3,
            'additional_color_scheme_default' => array_key_exists( 'additional_color_scheme_default', $opts ) ?
                $opts['additional_color_scheme_default'] : Color::COLOR_3
        ];


        if ( $style_opts['include_typography'] ) {
            $this->add_group_control(
                Group_Control_Typography::get_type(),
                [
                    'name' => $section_id . '_typography',
                    'label' => esc_html__('Typography', 'selection'),
                    'scheme' => Typography::TYPOGRAPHY_1,
                    'selector' => "{{WRAPPER}} .".$style_opts['typography_class'],
                ]
            );
        }

        $this->start_controls_tabs( $section_id.'_style_tabs' );

        $this->start_controls_tab(
            $section_id.'_normal_style_tab',
            ['label' => esc_html__( 'NORMAL', 'selection' )]
        );

        if ( $style_opts['include_color'] ) {
            $this->add_control(
                $section_id . '_normal_text_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );

        }

        if ( $style_opts['additional_color'] ) {
            $this->add_control(
                $section_id . '_' . $style_opts['additional_color_name'] . '_normal_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_color_name'], 'selection' ),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_color_class'] => 'color: {{VALUE}} !important;',

                    ],
                ]
            );
        }

        if ( $style_opts['include_bg'] ) {

            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_normal_background',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
                ]
            );

        }

        if ( $style_opts['include_css_filters'] ) {
            $this->add_group_control(
                Group_Control_Css_Filter::get_type(),
                [
                    'name' => 'css_filters_normal',
                    'selector' => '{{WRAPPER}} .'.$style_opts['html_class'],
                ]
            );
        }

        $this->add_control(
            $section_id . '_separate_normal',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_normal',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    $style_opts['additional_border_radius_selector'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab(
            $section_id.'_hover_style_tab',
            ['label' => esc_html__( 'HOVER', 'selection' )]
        );

        if ( $style_opts['include_color'] ) {
            $this->add_control(
                $section_id . '_hover_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_hover_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );
        }

        if ( $style_opts['additional_color_hover'] ) {
            $this->add_control(
                $section_id . '_' . $style_opts['additional_color_name'] . '_hover_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_color_name'], 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        $style_opts['additional_color_hover_selectors'] => 'color: {{VAlUE}} !important'
                    ],
                ]
            );
        }

        if ( $style_opts['include_bg'] ) {
            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_background_hover',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover",
                ]
            );
        }

        if ( $style_opts['include_css_filters'] ) {
            $this->add_group_control(
                Group_Control_Css_Filter::get_type(),
                [
                    'name' => 'css_filters_hover',
                    'selector' => '{{WRAPPER}} .'.$style_opts['html_class'].':hover',
                ]
            );
        }

        if ( $style_opts['include_transition'] ) {
            $this->add_control(
                $section_id.'_hover_transition',
                [
                    'label' => esc_html__( 'Hover transition duration', 'selection' ),
                    'type' => Controls_Manager::SLIDER,
                    'size_units' => [ 's' ],
                    'range' => [
                        's' => [
                            'min' => 0.1,
                            'max' => 5,
                            'step' => 0.1,
                        ],
                    ],
                    'default' => [
                        'unit' => 's',
                        'size' => 0,
                    ],
                    'selectors' => [
                        '{{WRAPPER}} .'.$style_opts['html_class'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};',
                        $style_opts['additional_transition_selector'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};;'
                    ],
                ]
            );
        }

        $this->add_control(
            $section_id.'_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_hover',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover",
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'].":hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    $style_opts['additional_border_radius_hover_selector'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover",
            ]
        );

        $this->end_controls_tab();

        if ( $style_opts['include_active_tab'] ) {

            $this->start_controls_tab(
                $section_id . '_focus_style_tab',
                ['label' => esc_html__( $style_opts['active_tab_name'], 'selection' )]
            );

            if ( $style_opts['include_color'] ) {
                $this->add_control(
                    $section_id . '_active_color',
                    [
                        'label' => esc_html__( $style_opts['color_prefix'] . 'Color', 'selection' ),
                        'type' => Controls_Manager::COLOR,
                        'scheme' => [
                            'type' => Color::get_type(),
                            'value' => $style_opts['color_scheme_default'],
                        ],
                        'selectors' => [
                            "{{WRAPPER}} ." . $style_opts['color_active_class'] => 'color: {{VALUE}} !important;',
                        ],
                    ]
                );
            }

            if ( $style_opts['additional_color'] ) {
                $this->add_control(
                    $section_id . '_' . $style_opts['additional_color_name'] . '_active_text_color',
                    [
                        'label' => esc_html__( $style_opts['additional_color_name'], 'selection' ),
                        'type' => Controls_Manager::COLOR,
                        'scheme' => [
                            'type' => Color::get_type(),
                            'value' => $style_opts['additional_color_scheme_default'],
                        ],
                        'selectors' => [
                            "{{WRAPPER}} ." . $style_opts['additional_color_active_class'] => 'color: {{VALUE}} !important;',
                        ],
                    ]
                );
            }

            if ( $style_opts['include_bg'] ) {
                $this->add_group_control(
                    Group_Control_Background::get_type(),
                    [
                        'name' => $section_id . '_background_active',
                        'label' => esc_html__( 'Background type', 'selection' ),
                        'types' => ['classic', 'gradient', 'video'],
                        'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                    ]
                );
            }

            $this->add_control(
                $section_id . '_separate_active',
                [
                    'type' => Controls_Manager::DIVIDER,
                ]
            );

            $this->add_group_control(
                Group_Control_Border::get_type(),
                [
                    'name' => $section_id . '_border_active',
                    'label' => esc_html__( 'Border Type', 'selection' ),
                    'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                ]
            );

            $this->add_responsive_control(
                $section_id . '_border_radius_active',
                [
                    'label' => esc_html__( 'Border radius', 'selection' ),
                    'type' => Controls_Manager::DIMENSIONS,
                    'size_units' => ['px', '%', 'em'],
                    'selectors' => [
                        "{{WRAPPER}} ." . $style_opts['active_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    ],
                ]
            );

            $this->add_group_control(
                Group_Control_Box_Shadow::get_type(),
                [
                    'name' => $section_id . '_box_shadow_active',
                    'label' => esc_html__( 'Box Shadow', 'selection' ),
                    'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                ]
            );

            $this->end_controls_tab();

        }


        $this->end_controls_tabs();
    }

    /**
     * Add widget controls: Style -> Section Style Before Image.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_before_image() {

        $this->start_controls_section( 'section_style_before_image', [
            'label' => esc_html__( 'Before image', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

         $this->generate_margin_padding_controls(
                 'section_style_before_image',
                 'mdp-comparimager-elementor-img-before'
         );

        $this->add_control(
            'before_image_object_fit',
            [
                'label' => esc_html__( 'Object fit', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'cover',
                'options' => [
                    'cover'  => esc_html__( 'Cover', 'selection' ),
                    'contain' => esc_html__( 'Contain', 'selection' ),
                    'fill' => esc_html__( 'Fill', 'selection' ),
                    'none' => esc_html__( 'None', 'selection' ),
                ],
                'separator' => 'before',
                'selectors' => [
                    '{{WRAPPER}} .mdp-comparimager-elementor-img-before img' => 'object-fit: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'before_image_object_fit_divider',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

         $this->generate_typography_tabs_controls( 'section_style_before_image', [
             'html_class' => 'mdp-comparimager-elementor-img-before',
             'include_color' => false,
             'include_typography' => false,
             'additional_transition_selector' => '{{WRAPPER}} .mdp-comparimager-elementor-label-before'
         ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style After Image.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_after_image() {

        $this->start_controls_section( 'section_style_after_image', [
            'label' => esc_html__( 'After image', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

         $this->generate_margin_padding_controls(
                 'section_style_after_image',
                 'mdp-comparimager-elementor-img-after'
         );


        $this->add_control(
            'after_image_object_fit',
            [
                'label' => esc_html__( 'Object fit', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'cover',
                'options' => [
                    'cover'  => esc_html__( 'Cover', 'selection' ),
                    'contain' => esc_html__( 'Contain', 'selection' ),
                    'fill' => esc_html__( 'Fill', 'selection' ),
                    'none' => esc_html__( 'None', 'selection' ),
                ],
                'separator' => 'before',
                'selectors' => [
                    '{{WRAPPER}} .mdp-comparimager-elementor-img-after img' => 'object-fit: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'after_image_object_fit_divider',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

         $this->generate_typography_tabs_controls( 'section_style_after_image', [
             'html_class' => 'mdp-comparimager-elementor-img-after',
             'include_color' => false,
             'include_typography' => false,
             'additional_transition_selector' => '{{WRAPPER}} .mdp-comparimager-elementor-label-after'
         ] );

        $this->end_controls_section();

    }


    /**
     * Add widget controls: Style -> Section Style Label Before.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_label_before() {

        $this->start_controls_section( 'section_style_label_before', [
            'label' => esc_html__( 'Label before', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

         $this->generate_margin_padding_controls(
                 'section_style_label_before',
                 'mdp-comparimager-elementor-label-before'
         );


        $this->add_responsive_control(
            'before_label_offset_x',
            [
                'label' => esc_html__( 'Label offset x', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'separator' => 'before',
                'selectors' => [
                    '{{WRAPPER}} .mdp-comparimager-elementor-label-before' => 'left: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'before_label_offset_y',
            [
                'label' => esc_html__( 'Label offset y', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} .mdp-comparimager-elementor-label-before' => 'top: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

         $this->generate_typography_tabs_controls( 'section_style_label_before', [
             'html_class' => 'mdp-comparimager-elementor-label-before',
             'include_color' => true,
             'typography_class' => 'mdp-comparimager-elementor-label-before',
             'color_class' => 'mdp-comparimager-elementor-label-before',
             'color_hover_class' => 'mdp-comparimager-elementor-label-before:hover'
         ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Label After.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_label_after() {

        $this->start_controls_section( 'section_style_label_after', [
            'label' => esc_html__( 'Label after', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

         $this->generate_margin_padding_controls(
                 'section_style_label_after',
                 'mdp-comparimager-elementor-label-after'
         );


        $this->add_responsive_control(
            'after_label_offset_x',
            [
                'label' => esc_html__( 'Label offset x', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'separator' => 'before',
                'selectors' => [
                    '{{WRAPPER}} .mdp-comparimager-elementor-label-after' => 'right: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'after_label_offset_y',
            [
                'label' => esc_html__( 'Label offset y', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} .mdp-comparimager-elementor-label-after' => 'top: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

         $this->generate_typography_tabs_controls( 'section_style_label_after', [
             'html_class' => 'mdp-comparimager-elementor-label-after',
             'include_color' => true,
             'typography_class' => 'mdp-comparimager-elementor-label-after',
             'color_class' => 'mdp-comparimager-elementor-label-after',
             'color_hover_class' => 'mdp-comparimager-elementor-label-after:hover'
         ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Overlay.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_overlay() {
        $this->start_controls_section( 'section_style_overlay', [
            'label' => esc_html__( 'Overlay', 'selection' ),
            'tab' => Controls_Manager::TAB_STYLE,
            'condition' => [
                'show_overlay' => 'yes'
            ]
        ] );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'background',
                'label' => esc_html__( 'Background', 'selection' ),
                'types' => [ 'classic', 'gradient' ],
                'selector' => '{{WRAPPER}} .mdp-comparimager-elementor-overlay',
            ]
        );



        $this->add_control(
            'overlay_blend_mode',
            [
                'label' => esc_html__( 'Blend Mode', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    '' => esc_html__( 'Normal', 'selection' ),
                    'multiply' => 'Multiply',
                    'screen' => 'Screen',
                    'overlay' => 'Overlay',
                    'darken' => 'Darken',
                    'lighten' => 'Lighten',
                    'color-dodge' => 'Color Dodge',
                    'saturation' => 'Saturation',
                    'color' => 'Color',
                    'difference' => 'Difference',
                    'exclusion' => 'Exclusion',
                    'hue' => 'Hue',
                    'luminosity' => 'Luminosity',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-comparimager-elementor-overlay' => 'mix-blend-mode: {{VALUE}}',
                ],
                'separator' => 'none',
            ]
        );

        $this->add_control(
            'overlay_hover_transition',
            [
                'label' => esc_html__( 'Hover transition duration', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 0,
                ],
                'condition' => [
                    'show_overlay_on' => 'hover'
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-comparimager-elementor-overlay' => 'transition: opacity {{SIZE}}{{UNIT}}',
                ],
            ]
        );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Overlay.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_compare_line() {
        $this->start_controls_section( 'section_style_compare_line', [
            'label' => esc_html__('Compare line', 'selection'),
            'tab' => Controls_Manager::TAB_STYLE
        ] );

        $this->add_responsive_control(
            'compare_line_width',
            [
                'label' => esc_html__( 'Compare line width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-comparimager-elementor-position-vertical .mdp-comparimager-elementor-compare-line' => 'width: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-comparimager-elementor-position-horizontal .mdp-comparimager-elementor-compare-line' => 'height: {{SIZE}}{{UNIT}};'
                ],
            ]
        );

        $this->add_control(
            'compare_line_color',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .mdp-comparimager-elementor-compare-line' => 'background-color: {{VALUE}}',
                ],
            ]
        );


        $this->end_controls_section();
    }

    /**
     * Add widget controls: Style -> Section Style Line Button.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_compare_line_button() {
        $this->start_controls_section( 'section_style_compare_line_button', [
            'label' => esc_html__('Compare line button', 'selection'),
            'tab' => Controls_Manager::TAB_STYLE
        ] );


        $this->add_responsive_control(
            'section_style_compare_line_button_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'separator' => 'after',
                'selectors' => [
                    "{{WRAPPER}} .mdp-comparimager-elementor-compare-line-button" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'before_icon_size',
            [
                'label' => esc_html__( 'Icon before size', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-comparimager-elementor-icon-before' => 'height: {{SIZE}}{{UNIT}}; width: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-comparimager-elementor-icon-before i' => 'font-size: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-comparimager-elementor-icon-before svg' => 'height: {{SIZE}}{{UNIT}}; width: {{SIZE}}{{UNIT}};'
                ],
            ]
        );

        $this->add_responsive_control(
            'after_icon_size',
            [
                'label' => esc_html__( 'Icon after size', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-comparimager-elementor-icon-after' => 'height: {{SIZE}}{{UNIT}}; width: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-comparimager-elementor-icon-after i' => 'font-size: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-comparimager-elementor-icon-after svg' => 'height: {{SIZE}}{{UNIT}}; width: {{SIZE}}{{UNIT}};'
                ],
            ]
        );

        $this->add_responsive_control(
            'space_between_icons',
            [
                'label' => esc_html__( 'Space between icons', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 300,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} .mdp-comparimager-elementor-position-vertical .mdp-comparimager-elementor-icon-before' => 'margin-right: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-comparimager-elementor-position-horizontal .mdp-comparimager-elementor-icon-before' => 'margin-bottom: {{SIZE}}{{UNIT}};',
                ],
            ]
        );


        $this->generate_typography_tabs_controls( 'section_style_compare_line_button', [
            'html_class' => 'mdp-comparimager-elementor-compare-line-button',
            'include_color' => true,
            'typography' => false,
            'additional_color' => true,
            'color_prefix' => 'Before Icon ',
            'additional_color_name' => 'After Icon Color',
            'additional_color_class' => 'mdp-comparimager-elementor-icon-after',
            'color_class' => 'mdp-comparimager-elementor-icon-before',
            'color_hover_class' => 'mdp-comparimager-elementor-compare-line-button:hover .mdp-comparimager-elementor-icon-before',
            'additional_color_hover_selectors' => '{{WRAPPER}} .mdp-comparimager-elementor-compare-line-button:hover .mdp-comparimager-elementor-icon-after',
            'additional_transition_selector' => '{{WRAPPER}} .mdp-comparimager-elementor-icon-before, {{WRAPPER}} .mdp-comparimager-elementor-icon-after'
        ] );

    }

    /**
     * Create handle button icon.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_handle_button_icon( $icon ) {

        if ( empty( $icon ) ) { return ''; }

        return $icon['library'] === 'svg' ?
            Icons_Manager::render_uploaded_svg_icon( $icon['value'] ) :
            '<i class="'.esc_attr( $icon['value'] ).'"></i>';
    }

    /**
     * Create image block method.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_image_block( $suffix, $image, $settings ) {
        return sprintf(
            '<div class="mdp-comparimager-elementor-img-%s">%s</div>',
                    $suffix,
                    Group_Control_Image_Size::get_attachment_image_html( $settings, 'images_size', $image )
        );
    }

    /**
     * Create image label.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_label( $suffix, $text ) {
        return sprintf(
                '<div class="mdp-comparimager-elementor-label mdp-comparimager-elementor-label-%s">%s</div>',
                        esc_attr( $suffix ),
                        esc_html( $text )
        );
    }

    /**
     * Create image slider method.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_image_slider( $settings ) {
        $before_image = $this->create_image_block( 'before', 'before_image',$settings );
        $after_image = $this->create_image_block( 'after', 'after_image',$settings );
        $handle = $this->create_handle( $settings );
        $label_before = $this->create_label( 'before', $settings['before_image_label'] );
        $label_after = $this->create_label( 'after', $settings['after_image_label'] );
        $overlay = $settings['show_overlay'] === 'yes' ?
            sprintf(
            '<div class="mdp-comparimager-elementor-overlay %s"></div>',
            $settings['show_overlay_on'] === 'hover' ? 'mdp-comparimager-elementor-overlay-hover' : ''
            ) :
            '';

        return sprintf(
            '<div class="mdp-comparimager-elementor-comparison-slider">%s %s %s %s %s %s</div>',
                    $before_image,
                    $after_image,
                    $handle,
                    $label_before,
                    $label_after,
                    $overlay
        );
    }

    /**
     * Create handle method.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_handle( $settings ) {
        $next_icon = $settings['line_position'] === 'horizontal' ?
            $settings['horizontal_icon_next'] :
            $settings['vertical_icon_next'];
        $previous_icon = $settings['line_position'] === 'horizontal' ?
            $settings['horizontal_icon_previous'] :
            $settings['vertical_icon_previous'];
        $previous_button_icon = $this->create_handle_button_icon( $previous_icon );
        $next_button_icon = $this->create_handle_button_icon( $next_icon );

       return sprintf(
        '<div class="mdp-comparimager-elementor-compare-handle">
                    <div class="mdp-comparimager-elementor-compare-line">
                        <div class="mdp-comparimager-elementor-compare-line-button">
                            <span class="mdp-comparimager-elementor-icon-before">%s</span>
                            <span class="mdp-comparimager-elementor-icon-after">%s</span>
                        </div>
                    </div>
                </div>',
                $previous_button_icon,
                $next_button_icon
        );
    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {
        $settings = $this->get_settings_for_display();

        echo sprintf(
            '<!-- Start Comparimager for Elementor WordPress Plugin -->
                    <div class="mdp-comparimager-elementor-box mdp-comparimager-elementor-position-%s"
                    data-move-handle="%s"
                    data-line-position="%s"
                    >
                        %s
                    </div>
                    <!-- End Comparimager for Elementor WordPress Plugin -->',
                    esc_attr( $settings['line_position'] ),
                    esc_attr( $settings['move_handle'] ),
                    esc_attr( $settings['line_position'] ),
                    $this->create_image_slider( $settings )
        );

    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/comparimager';

    }

}
