<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Imager for Elementor
 * Customizable image grid with nice hover effects for Elementor
 * Exclusively on https://1.envato.market/imager-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Cherviakov Vlad (vladchervjakov@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Css_Filter;
use Elementor\Group_Control_Text_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Plugin;
use Elementor\Repeater;
use Elementor\Utils;
use Elementor\Core\Schemes\Typography;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;
use Merkulove\Selection\Unity\ElementorControls;

/** @noinspection PhpUnused */
/**
 * Imager - Custom Elementor Widget.
 **/
class imager_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style(
            'mdp-selection-elementor-admin',
            UnityPlugin::get_url().'css/elementor-admin'.UnityPlugin::get_suffix().'.css',
            [],
            UnityPlugin::get_version()
        );
        wp_register_style(
            'mdp-imager-elementor',
            UnityPlugin::get_url().'css/imager-elementor'.UnityPlugin::get_suffix().'.css',
            [],
            UnityPlugin::get_version()
        );

	    wp_register_script(
            'tilt-jquery',
            UnityPlugin::get_url().'js/tilt.jquery'.UnityPlugin::get_suffix().'.js',
            ['jquery'],
            UnityPlugin::get_version(),
            true
        );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {
        return 'mdp-imager-elementor';
    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Image Box', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {
        return 'mdp-imager-elementor-widget-icon';
    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {
        return [ 'selection-category' ];
    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {
        return [
            'Merkulove',
            'Image Box',
            'image',
            'box',
            'grid',
            'reveal',
            'hover',
            'products',
            'woocommerce',
            'blog',
            'post',
            'archive'
        ];
    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {
        return [
            'mdp-imager-elementor',
            'mdp-selection-elementor-admin'
        ];
    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {
		return [
            'jquery',
            'tilt-jquery'
        ];
    }

    /**
     * We write the existing categories of posts into an array.
     */
    public function get_post_category() {

        $wp_cat = get_categories('&hide_empty=0');

        /** If no category is selected.  */
        $options['select_category'] = 'Select category';

        /** Add available post categories. */
        foreach( $wp_cat as $cat_item ){
            $options[$cat_item->slug] =  $cat_item->name;
        }

        return $options;
    }

    /**
     * We write the existing product categories into an array.
     */
    public function get_products_category() {

        $wp_cat = get_categories( array(
            'taxonomy'     => 'product_cat',
            'orderby'      => 'name',
            'pad_counts'   => false,
            'hierarchical' => 1,
            'hide_empty'   => false
        ) );

        /** If no category is selected.  */
        $options['select_category'] = 'Select category';

        /** Add available product categories. */
        foreach( $wp_cat as $cat_item ){
            $options[$cat_item->slug] =  $cat_item->name;
        }

        return $options;
    }

    /**
     * A group of controllers for setting styles.
     *
     * @param $name_Section - The name of the styles section.
     * @param string $css_class - Specifies one or more class names for an element.
     * @param array $condition - Interconnected controllers.
     * @param array $display - Determines which controllers to display. {
     *      @type $key 'margin'             => (boolean) - Use margin.
     *      @type $key 'padding'            => (boolean) - Use padding.
     *      @type $key 'color'              => (boolean) - Use color.
     *      @type $key 'typography'         => (boolean) - Use typography.
     *      @type $key 'text-shadow'        => (boolean) - Use text shadow.
     *      @type $key 'alignment'          => (boolean) - Use alignment.
     *      @type $key 'alignment-justify'  => (boolean) - Use alignment justify.
     *      @type $key 'background'         => (boolean) - Use background.
     *      @type $key 'border'             => (boolean) - Use border.
     *      @type $key 'border-radius'      => (boolean) - Use border radius.
     *      @type $key 'box-shadow'         => (boolean) - Use box shadow.
     *      @type $key 'css-filter'         => (boolean) - Use css filter.
     *
     * }
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     */
    public function get_settings_style_group( $name_Section, $css_class, $condition = [], $display = [] ) {

        /** Margin. */
        if ( isset($display['margin']) && $display['margin'] === true ) {
            $this->add_responsive_control(
                'avatar_' . $name_Section . '_margin',
                [
                    'label'      => esc_html__( 'Margin', 'avatar-elementor' ),
                    'type'       => Controls_Manager::DIMENSIONS,
                    'size_units' => [ 'px', '%', 'em' ],
                    'selectors'  => [
                        '{{WRAPPER}} ' . $css_class => 'margin: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                    ],
                    'toggle'     => true,
                    'condition'  => $condition,
                ]
            );
        }

        /** Padding. */
        if ( isset($display['padding']) && $display['padding'] === true ) {
            $this->add_responsive_control(
                'avatar_' . $name_Section . '_padding',
                [
                    'label'      => esc_html__( 'Padding', 'avatar-elementor' ),
                    'type'       => Controls_Manager::DIMENSIONS,
                    'size_units' => [ 'px', '%', 'em' ],
                    'selectors'  => [
                        '{{WRAPPER}} ' . $css_class => 'padding: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                    ],
                    'toggle'     => true,
                    'separator'  => 'after',
                    'condition'  => $condition,
                ]
            );
        }

        /** Text Color. */
        if ( isset($display['color']) && $display['color'] === true ) {
            $this->add_control(
                'avatar_' . $name_Section . '_text_color',
                [
                    'label'     => esc_html__( 'Color', 'avatar-elementor' ),
                    'type'      => Controls_Manager::COLOR,
                    'selectors' => [
                        '{{WRAPPER}} ' . $css_class => 'color: {{VALUE}}',
                    ],
                    'condition' => $condition,
                ]
            );
        }

        /** Typography */
        if ( isset($display['typography']) && $display['typography'] === true ) {
            $this->add_group_control(
                Group_Control_Typography::get_type(),
                [
                    'name'      => 'avatar_' . $name_Section . '_typography',
                    'label'     => esc_html__( 'Typography', 'avatar-elementor' ),
                    'scheme'    => Typography::TYPOGRAPHY_1,
                    'selector'  => '{{WRAPPER}} ' . $css_class,
                    'condition' => $condition,
                ]
            );
        }

        /** Text Shadow. */
        if ( isset($display['text-shadow']) && $display['text-shadow'] === true ) {
            $this->add_group_control(
                Group_Control_Text_Shadow::get_type(),
                [
                    'name'      => 'avatar_' . $name_Section . '_text_shadow',
                    'label'     => esc_html__( 'Text Shadow', 'avatar-elementor' ),
                    'selector'  => '{{WRAPPER}} ' . $css_class,
                    'condition' => $condition,
                ]
            );
        }

        /** Alignment. */
        if ( isset($display['alignment']) && $display['alignment'] === true ) {
            $this->add_responsive_control(
                'avatar_' . $name_Section . '_align',
                [
                    'label'     => esc_html__( 'Alignment', 'avatar-elementor' ),
                    'type'      => Controls_Manager::CHOOSE,
                    'options'   => [
                        'left'    => [
                            'title' => esc_html__( 'Left', 'avatar-eletilt-jquerymentor' ),
                            'icon'  => 'fa fa-align-left',
                        ],
                        'center'  => [
                            'title' => esc_html__( 'Center', 'avatar-elementor' ),
                            'icon'  => 'fa fa-align-center',
                        ],
                        'right'   => [
                            'title' => esc_html__( 'Right', 'avatar-elementor' ),
                            'icon'  => 'fa fa-align-right',
                        ],
                    ],
                    'default'   => 'center',
                    'selectors' => [
                        '{{WRAPPER}} ' . $css_class => 'text-align: {{header_align}};',
                    ],
                    'toggle'    => true,
                    'condition' => $condition,
                ]
            );
        }

        /** Alignment justify. */
        if ( isset($display['alignment-justify']) && $display['alignment-justify'] === true ) {
            $this->add_responsive_control(
                'avatar_' . $name_Section . '_align_justify',
                [
                    'label'     => esc_html__( 'Alignment', 'avatar-elementor' ),
                    'type'      => Controls_Manager::CHOOSE,
                    'options'   => [
                        'left'    => [
                            'title' => esc_html__( 'Left', 'avatar-elementor' ),
                            'icon'  => 'fa fa-align-left',
                        ],
                        'center'  => [
                            'title' => esc_html__( 'Center', 'avatar-elementor' ),
                            'icon'  => 'fa fa-align-center',
                        ],
                        'right'   => [
                            'title' => esc_html__( 'Right', 'avatar-elementor' ),
                            'icon'  => 'fa fa-align-right',
                        ],
                        'justify' => [
                            'title' => esc_html__( 'Justify', 'avatar-elementor' ),
                            'icon'  => 'fa fa-align-justify',
                        ],
                    ],
                    'default'   => 'center',
                    'selectors' => [
                        '{{WRAPPER}} ' . $css_class => 'text-align: {{header_align}};',
                    ],
                    'toggle'    => true,
                    'condition' => $condition,
                ]
            );
        }

        /** Background. */
        if ( isset($display['background']) && $display['background'] === true ) {
            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name'      => 'avatar_' . $name_Section . '_background',
                    'label'     => esc_html__( 'Background', 'avatar-elementor' ),
                    'types'     => [ 'classic', 'gradient' ],
                    'separator' => 'before',
                    'selector'  => '{{WRAPPER}} ' . $css_class,
                    'condition' => $condition,
                ]
            );
        }

        /** Border. */
        if ( isset($display['border']) && $display['border'] === true ) {
            $this->add_group_control(
                Group_Control_Border::get_type(),
                [
                    'name'      => 'avatar_' . $name_Section . '_border',
                    'label'     => esc_html__( 'Border', 'avatar-elementor' ),
                    'selector'  => '{{WRAPPER}} ' . $css_class,
                    'separator' => 'before',
                    'condition' => $condition,
                ]
            );
        }

        /** Border Radius. */
        if ( isset($display['border-radius']) && $display['border-radius'] === true ) {
            $this->add_responsive_control(
                'avatar_' . $name_Section . '_border_radius',
                [
                    'label'      => esc_html__( 'Border Radius', 'avatar-elementor' ),
                    'type'       => Controls_Manager::DIMENSIONS,
                    'size_units' => [ 'px', '%', 'em' ],
                    'selectors'  => [
                        '{{WRAPPER}} ' . $css_class => 'border-radius: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                    ],
                    'separator'  => 'after',
                    'toggle'     => true,
                    'condition'  => $condition,
                ]
            );
        }

        /** Box Shadow. */
        if ( isset($display['box-shadow']) && $display['box-shadow'] === true ) {
            $this->add_group_control(
                Group_Control_Box_Shadow::get_type(),
                [
                    'name'      => 'avatar_' . $name_Section . '_box_shadow',
                    'label'     => esc_html__( 'Box Shadow', 'avatar-elementor' ),
                    'selector'  => '{{WRAPPER}} ' . $css_class,
                    'condition' => $condition,
                ]
            );
        }

        /** CSS filter. */
        if ( isset($display['css-filter']) && $display['css-filter'] === true ) {
            $this->add_group_control(
                Group_Control_Css_Filter::get_type(),
                [
                    'name'     => 'avatar_' . $name_Section . '_css_filter',
                    'label'    => esc_html__( 'CSS filter', 'avatar-elementor' ),
                    'selector' => '{{WRAPPER}} ' . $css_class,
                    'condition' => $condition,
                ]
            );
        }
    }


    /**
     * Settings for the general tab.
     */
    public function get_general_content_controls() {

        /** Start section */
        $this->start_controls_section(
            'general_content',
            [
                'label' => esc_html__( 'General', 'selection' ),
                'tab' => Controls_Manager::TAB_CONTENT,
            ]
        );

        $this->add_responsive_control(
            'general_columns',
            [
                'label'           => esc_html__( 'Columns', 'selection' ),
                'type'            => Controls_Manager::SLIDER,
                'size_units'      => [ 'px' ],
                'devices'         => [ 'desktop', 'tablet', 'mobile' ],
                'range'           => [
                    'px' => [
                        'min'  => 1,
                        'max'  => 10,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 2,
                ],
                'selectors'       => [
                    '{{WRAPPER}} .mdp-imager-wrap' => 'width: calc(100% / {{size}});',
                ],
            ]
        );

        $this->add_control(
            'general_source',
            [
                'label' => esc_html__( 'Source', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'custom',
                'options' => [
                    'custom'  => esc_html__( 'Custom', 'selection' ),
                    'posts' => esc_html__( 'Posts', 'selection' ),
                    'woocommerce' => esc_html__( 'WooCommerce', 'selection' ),
                ],
            ]
        );

        /** Post category. */
        $this->add_control(
            'general_posts_category',
            [
                'label' => esc_html__( 'Category', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'select_category',
                'options' => $this->get_post_category(),
                'condition' => [ 'general_source' => 'posts' ]
            ]
        );

        /** Products category. */
        $this->add_control(
            'general_products_category',
            [
                'label' => esc_html__( 'Category', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'select_category',
                'options' => $this->get_products_category(),
                'condition' => [ 'general_source' => 'woocommerce' ]
            ]
        );

        $this->add_control(
            'general_style',
            [
                'label' => esc_html__( 'Type', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'overlay',
                'options' => [
                    'overlay'  => esc_html__( 'Overlay', 'selection' ),
                    'default'  => esc_html__( 'Card', 'selection' ),
                ],
            ]
        );

        $this->add_control(
            'imager_hover',
            [
                'label' => esc_html__( 'Image Hover', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => '',
                'options' => [
                    ''  => esc_html__( 'Default', 'selection' ),
                    'mdp-imager-blur'  => esc_html__( 'Blur', 'selection' ),
                    'mdp-imager-zoom'  => esc_html__( 'Zoom', 'selection' ),
                    'mdp-imager-swing'  => esc_html__( 'Swing', 'selection' ),
                    'mdp-imager-grayscale'  => esc_html__( 'Grayscale', 'selection' ),
                    'mdp-imager-color'  => esc_html__( 'Color', 'selection' ),
                    'mdp-imager-contrast'  => esc_html__( 'Contrast', 'selection' ),
                    'mdp-imager-jello'  => esc_html__( 'Jello', 'selection' ),
                    'mdp-imager-bounce'  => esc_html__( 'Bounce', 'selection' ),
                    'mdp-imager-flip'  => esc_html__( 'Flip', 'selection' ),
                    'mdp-imager-sepia'  => esc_html__( 'Sepia', 'selection' ),
                ],
                'separator' => 'before'
            ]
        );

        $this->add_control(
            'imager_hover_origin',
            [
                'label' => esc_html__( 'Transform Origin', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'center center',
                'options' => [
                    'center center' => esc_html__( 'Center Center', 'selection' ),
                    'center left' => esc_html__( 'Center Left', 'selection' ),
                    'center right' => esc_html__( 'Center Right', 'selection' ),
                    'top center' => esc_html__( 'Top Center', 'selection' ),
                    'top left' => esc_html__( 'Top Left', 'selection' ),
                    'top right' => esc_html__( 'Top Right', 'selection' ),
                    'bottom center' => esc_html__( 'Bottom Center', 'selection' ),
                    'bottom left' => esc_html__( 'Bottom Left', 'selection' ),
                    'bottom right' => esc_html__( 'Bottom Right', 'selection' ),
                ],
                'selectors'       => [
                    '{{WRAPPER}} .mdp-imager-image' => 'transform-origin: {{value}};',
                ],
                'condition' => [ 'imager_hover' => 'mdp-imager-zoom' ],
            ]
        );

        $this->add_control(
            'imager_hover_duration',
            [
                'label'           => esc_html__( 'Hover Animation Duration', 'selection' ),
                'type'            => Controls_Manager::SLIDER,
                'size_units'      => [ 'px' ],
                'range'           => [
                    'px' => [
                        'min'  => 0,
                        'max'  => 5,
                        'step' => .1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => .8,
                ],
                'selectors'       => [
                    '{{WRAPPER}} .mdp-imager-wrap .mdp-imager-box-image' => 'animation-duration: {{size}}s;',
                    '{{WRAPPER}} .mdp-imager-wrap:hover .mdp-imager-image' => 'animation-duration: {{size}}s;',
                    '{{WRAPPER}} .mdp-imager-wrap:not(:hover) .mdp-imager-image' => 'animation-duration: calc({{size}}s / 2);',

                ],
                'conditions' => [
                    'relation' => 'or',
                    'terms' => [
                        [
                            'name' => 'imager_hover',
                            'operator' => '!==',
                            'value' => '',
                        ],
                    ],
                ],
            ]
        );

        $this->add_control(
            'imager_hover_delay',
            [
                'label'           => esc_html__( 'Hover Animation Delay', 'selection' ),
                'type'            => Controls_Manager::SLIDER,
                'size_units'      => [ 'px' ],
                'range'           => [
                    'px' => [
                        'min'  => 0,
                        'max'  => 5,
                        'step' => .1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 0,
                ],
                'selectors'       => [
                    '{{WRAPPER}} .mdp-imager-wrap .mdp-imager-box-image' => 'animation-delay: {{size}}s;',
                    '{{WRAPPER}} .mdp-imager-wrap:hover .mdp-imager-image' => 'animation-delay: {{size}}s;',
                    '{{WRAPPER}} .mdp-imager-wrap:not(:hover) .mdp-imager-image' => 'animation-delay: calc({{size}}s / 2);',

                ],
                'conditions' => [
                    'relation' => 'or',
                    'terms' => [
                        [
                            'name' => 'imager_hover',
                            'operator' => '!==',
                            'value' => '',
                        ],
                    ],
                ],
            ]
        );

        $this->add_control(
            'imager_details_hover',
            [
                'label' => esc_html__( 'Text Hover', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'mdp-imager-details-default',
                'options' => [
                    'mdp-imager-details-default'  => esc_html__( 'Default', 'selection' ),
                    'mdp-imager-details-flip'  => esc_html__( 'Flip', 'selection' ),
                    'mdp-imager-details-slide'  => esc_html__( 'Slide', 'selection' ),
                    'mdp-imager-details-zoom'  => esc_html__( 'Zoom', 'selection' ),
                ],
                'condition' => [ 'general_style' => 'overlay' ]
            ]
        );

        $this->add_control(
            'imager_details_hover_direction',
            [
                'label' => esc_html__( 'Text Animation Side', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'left',
                'options' => [
                    'left'  => esc_html__( 'Left', 'selection' ),
                    'right'  => esc_html__( 'Right', 'selection' ),
                    'top'  => esc_html__( 'Top', 'selection' ),
                    'bottom'  => esc_html__( 'Bottom', 'selection' ),
                ],
                'conditions' => [
                    'relation' => 'and',
                    'terms' => [
                        [
                            'name' => 'general_style',
                            'operator' => '==',
                            'value' => 'overlay',
                        ],
                        [
                            'name' => 'imager_details_hover',
                            'operator' => '!==',
                            'value' => 'mdp-imager-details-default',
                        ],
                    ],
                ],
            ]
        );

        $this->add_control(
            'imager_details_hover_duration',
            [
                'label'           => esc_html__( 'Text Animation Duration', 'selection' ),
                'type'            => Controls_Manager::SLIDER,
                'size_units'      => [ 'px' ],
                'range'           => [
                    'px' => [
                        'min'  => 0,
                        'max'  => 5,
                        'step' => .1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => .8,
                ],
                'selectors'       => [
                    '{{WRAPPER}} .mdp-imager-wrap:hover .mdp-imager-box-overlay' => 'transition: {{size}}s;',
                    '{{WRAPPER}} .mdp-imager-wrap:not(:hover) .mdp-imager-box-overlay' => 'transition: calc({{size}}s / 2);',
                    '{{WRAPPER}} .mdp-imager-wrap:hover .mdp-imager-box-overlay > *' => 'transition: {{size}}s;',
                    '{{WRAPPER}} .mdp-imager-wrap:not(:hover) .mdp-imager-box-overlay > *' => 'transition: calc({{size}}s / 2);',
                ],
                'conditions' => [
                    'relation' => 'and',
                    'terms' => [
                        [
                            'name' => 'general_style',
                            'operator' => '==',
                            'value' => 'overlay',
                        ],
                        [
                            'name' => 'imager_details_hover',
                            'operator' => '!==',
                            'value' => 'mdp-imager-details-default',
                        ],
                    ],
                ],
            ]
        );

        $this->add_control(
            'imager_details_hover_delay',
            [
                'label'           => esc_html__( 'Text Animation Delay', 'selection' ),
                'type'            => Controls_Manager::SLIDER,
                'size_units'      => [ 'px' ],
                'range'           => [
                    'px' => [
                        'min'  => 0,
                        'max'  => 5,
                        'step' => .1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 0,
                ],
                'selectors'       => [
                    '{{WRAPPER}} .mdp-imager-wrap:hover .mdp-imager-box-overlay' => 'transition-delay: {{size}}s;',
                    '{{WRAPPER}} .mdp-imager-wrap:not(:hover) .mdp-imager-box-overlay' => 'transition-delay: calc({{size}}s / 2);',
                    '{{WRAPPER}} .mdp-imager-wrap:hover .mdp-imager-box-overlay > *' => 'transition-delay: {{size}}s;',
                    '{{WRAPPER}} .mdp-imager-wrap:not(:hover) .mdp-imager-box-overlay > *' => 'transition-delay: calc({{size}}s / 2);',
                ],
                'conditions' => [
                    'relation' => 'and',
                    'terms' => [
                        [
                            'name' => 'general_style',
                            'operator' => '==',
                            'value' => 'overlay',
                        ],
                        [
                            'name' => 'imager_details_hover',
                            'operator' => '!==',
                            'value' => 'mdp-imager-details-default',
                        ],
                    ],
                ],
            ]
        );

        $this->add_control(
            'general_show_animation',
            [
                'label' => esc_html__( 'Tilt on hover', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_responsive_control(
            'general_max_tilt',
            [
                'label'           => esc_html__( 'Max tilt', 'selection' ),
                'type'            => Controls_Manager::SLIDER,
                'size_units'      => [ 'px' ],
                'range'           => [
                    'px' => [
                        'min'  => 1,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 10,
                ],
                'condition' => [ 'general_show_animation' => 'yes' ]
            ]
        );

        $this->add_control(
            'general_link_type',
            [
                'label'   => esc_html__( 'Link Type', 'selection' ),
                'type'    => Controls_Manager::SELECT,
                'default' => 'none',
                'options' => [
                    'none'   => esc_html__( 'None', 'selection' ),
                    'box'    => esc_html__( 'Box', 'selection' ),
                    'title'  => esc_html__( 'Title', 'selection' ),
                ],
                'separator' => 'before',
            ]
        );

        /** End section */
        $this->end_controls_section();

    }

    /**
     * Settings for the layout tab.
     */
    public function get_layout_content_controls() {

        /** Start section */
        $this->start_controls_section(
            'layout_content',
            [
                'label' => esc_html__( 'Layout', 'selection' ),
                'tab' => Controls_Manager::TAB_CONTENT,
            ]
        );

        /** Alignment. */
        $this->add_responsive_control(
            'box_align',
            [
                'label'     => esc_html__( 'Alignment', 'selection' ),
                'type'      => Controls_Manager::CHOOSE,
                'options'   => [
                    'start'         => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon'  => 'fa fa-align-left',
                    ],
                    'center'        => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon'  => 'fa fa-align-center',
                    ],
                    'end'           => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon'  => 'fa fa-align-right',
                    ],
                    'space-between' => [
                        'title' => esc_html__( 'Space between', 'selection' ),
                        'icon'  => 'fa fa-expand-arrows-alt',
                    ],
                    'space-around'  => [
                        'title' => esc_html__( 'Space around', 'selection' ),
                        'icon'  => 'fa fa-compress-arrows-alt',
                    ],
                ],
                'default'   => 'space-around',
                'selectors' => [
                    '{{WRAPPER}} .mdp-imager-item' => 'justify-content: {{value}};',
                    '{{WRAPPER}} .mdp-imager-box-overlay' => 'justify-content: {{value}};',
                ],
                'toggle'    => true,
            ]
        );

        $layout = new Repeater();

        /** Item. */
        $layout->add_control(
            'layout_item',
            [
                'label'   => esc_html__( 'Item', 'selection' ),
                'type'    => Controls_Manager::SELECT,
                'default' => 'title',
                'options' => [
                    'image'       => esc_html__( 'Image', 'selection' ),
                    'title'       => esc_html__( 'Title', 'selection' ),
                    'description' => esc_html__( 'Description', 'selection' ),
                    'price'       => esc_html__( 'Price', 'selection' ),
                    'date'        => esc_html__( 'Date', 'selection' ),
                    'category'    => esc_html__( 'Category', 'selection' ),
                ],
            ]
        );

        /** Priority. */
        $this->add_control(
            'layout_priority',
            [
                'label'       => esc_html__( 'Priority', 'selection' ),
                'type'        => Controls_Manager::REPEATER,
                'fields'      => $layout->get_controls(),
                'default'     => [
                    [ 'layout_item' => 'image' ],
                    [ 'layout_item' => 'title' ],
                    [ 'layout_item' => 'description' ],
                ],
                'separator'   => 'before',
                'title_field' => 'Layout item  - {{{layout_item}}}'
            ]
        );

        /** End section */
        $this->end_controls_section();

    }

    /**
     * Settings for the ImageBox tab.
     */
    public function get_image_box_content_controls() {

        /** Start section */
        $this->start_controls_section(
            'imagebox_content',
            [
                'label' => esc_html__( 'Image Box', 'selection' ),
                'tab' => Controls_Manager::TAB_CONTENT,
                'condition' => [ 'general_source' => 'custom' ]
            ]
        );

        $imagebox = new Repeater();

        $imagebox->add_control(
            'imagebox_item_image',
            [
                'label'   => __( 'Choose Image', 'selection' ),
                'type'    => Controls_Manager::MEDIA,
                'default' => [
                    'url' => Utils::get_placeholder_image_src(),
                ],
            ]
        );

        /** Title. */
        $imagebox->add_control(
            'imagebox_item_title',
            [
                'label'       => esc_html__( 'Title', 'selection' ),
                'type'        => Controls_Manager::TEXT,
                'dynamic'     => [ 'active' => true ],
                'default'     => esc_html__( 'Title', 'selection' ),
                'placeholder' => esc_html__( 'Title', 'selection' ),
            ]
        );

        /** Description. */
        $imagebox->add_control(
            'imagebox_item_description',
            [
                'label'       => esc_html__( 'Description', 'selection' ),
                'type'        => Controls_Manager::TEXTAREA,
                'rows'        => 3,
                'default'     => esc_html__( 'Description', 'selection' ),
                'placeholder' => esc_html__( 'Description', 'selection' ),
            ]
        );

        /** Date */
        $imagebox->add_control(
            'imagebox_item_date',
            [
                'label' => esc_html__( 'Date', 'selection' ),
                'type' => Controls_Manager::DATE_TIME,
            ]
        );

        /** Show category. */
        $imagebox->add_control(
            'imagebox_show_category',
            [
                'label' => esc_html__( 'Show category', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        /** Category. */
        $imagebox->add_control(
            'imagebox_item_category',
            [
                'label'       => esc_html__( 'Category or Tag', 'selection' ),
                'type'        => Controls_Manager::TEXT,
                'dynamic'     => [ 'active' => true ],
                'default'     => esc_html__( 'Category or Tag', 'selection' ),
                'placeholder' => esc_html__( 'Category or Tag', 'selection' ),
                'condition' => [ 'imagebox_show_category' => 'yes']
            ]
        );

        /** Show link. */
        $imagebox->add_control(
            'imagebox_show_link',
            [
                'label' => esc_html__( 'Enable link', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        /** Link. */
        $imagebox->add_control(
            'imagebox_link_item',
            [
                'label'         => esc_html__( 'Link', 'selection' ),
                'type'          => Controls_Manager::URL,
                'placeholder'   => esc_html__( 'https://codecanyon.net/user/merkulove', 'selection' ),
                'show_external' => true,
                'default'       => [
                    'url'         => esc_url( '#' ),
                    'is_external' => true,
                    'nofollow'    => true,
                ],
                'condition' => [ 'imagebox_show_link' => 'yes']
            ]
        );

        $this->add_control(
            'imagebox_item',
            [
                'label'       => esc_html__( 'Item', 'selection' ),
                'type'        => Controls_Manager::REPEATER,
                'fields'      => $imagebox->get_controls(),
                'separator'   => 'before',
                'default' => [
                    [
                        'imagebox_item_image' => '',
                        'imagebox_item_title' => esc_html__( 'Title', 'selection' ),
                        'imagebox_item_description' =>  esc_html__( 'Description', 'selection' ),
                        'imagebox_show_link' => 'yes',
                        'imagebox_link_item'    => '#'
                    ],
                    [
                        'imagebox_item_image' => '',
                        'imagebox_item_title' => esc_html__( 'Title 2', 'selection' ),
                        'imagebox_item_description' =>  esc_html__( 'Description 2', 'selection' ),
                        'imagebox_show_link' => 'yes',
                        'imagebox_link_item'    => '#'
                    ]
                ],
                'title_field' => 'Item  - {{{imagebox_item_title}}}'
            ]
        );

        /** End section */
        $this->end_controls_section();

    }

    /**
     * Settings for the grid style tab.
     */
    public function get_grid_style_controls() {

        /** Start section */
        $this->start_controls_section( 'style_grid',
            [
                'label' => esc_html__( 'Grid', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE,
            ] );

        /** Column spacing */
        $this->add_responsive_control(
            'grid_сolumn_spacing',
            [
                'label' => esc_html__( 'Column spacing', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ '%', 'px' ],
                'range' => [
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ],
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 25,
                ],
                'selectors'  => [
                    '{{WRAPPER}} .mdp-imager-wrap'      => 'padding-left: calc({{size}}{{unit}} / 2); padding-right: calc({{size}}{{unit}} / 2);',
                    '{{WRAPPER}} .mdp-imager-container' => 'margin-left: calc(-{{size}}{{unit}} / 2); margin-right: calc(-{{size}}{{unit}} / 2);',
                ],
            ]
        );

        /** Row spacing. */
        $this->add_responsive_control(
            'grid_row_spacing',
            [
                'label' => esc_html__( 'Row spacing', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ '%', 'px' ],
                'range' => [
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ],
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 25,
                ],
                'separator' => 'after',
                'selectors'  => [
                    '{{WRAPPER}} .mdp-imager-wrap' => 'margin-bottom: {{size}}{{unit}};',
                ],
            ]
        );

        $this->get_settings_style_group(
            'grid',
            '.mdp-imager-box-grid',
            [],
            [
                'margin' => true,
                'padding' => true,
                'color' => true,
                'typography' => true,
                'text-shadow' => true,
                'alignment-justify' => true,
                'background' => true,
                'border' => true,
                'border-radius' => true,
                'box-shadow' => true,
            ]
        );

        /** End section */
        $this->end_controls_section();

    }

    /**
     * Settings for the title style tab.
     */
    public function get_title_style_controls() {

        /** Start section */
        $this->start_controls_section( 'style_title',
            [
                'label' => esc_html__( 'Title', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE,
            ] );

        /** Width. */
        $this->add_responsive_control(
            'title_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ '%', 'px' ],
                'range' => [
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ],
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 100,
                ],
                'separator' => 'after',
                'selectors'  => [
                    '{{WRAPPER}} .mdp-imager-box-title' => 'width: {{size}}{{unit}};',
                ],
            ]
        );

        $this->get_settings_style_group(
            'title',
            '.mdp-imager-box-title',
            [],
            [
                'margin' => true,
                'padding' => true,
                'color' => true,
                'typography' => true,
                'text-shadow' => true,
                'alignment-justify' => true,
                'background' => true,
                'border' => true,
                'border-radius' => true,
                'box-shadow' => true,
            ]
        );

        /** End section */
        $this->end_controls_section();

    }

    /**
     * Settings for the description style tab.
     */
    public function get_description_style_controls() {

        /** Start section */
        $this->start_controls_section( 'style_description',
            [
                'label' => esc_html__( 'Description', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE,
            ] );

        /** Width. */
        $this->add_responsive_control(
            'description_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ '%', 'px' ],
                'range' => [
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ],
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 100,
                ],
                'separator' => 'after',
                'selectors'  => [
                    '{{WRAPPER}} .mdp-imager-box-description' => 'width: {{size}}{{unit}};',
                ],
            ]
        );

        $this->get_settings_style_group(
            'description',
            '.mdp-imager-box-description',
            [],
            [
                'margin' => true,
                'padding' => true,
                'color' => true,
                'typography' => true,
                'text-shadow' => true,
                'alignment-justify' => true,
                'background' => true,
                'border' => true,
                'border-radius' => true,
                'box-shadow' => true,
            ]
        );

        /** End section */
        $this->end_controls_section();

    }

    /**
     * Settings for the price style tab.
     */
    public function get_price_style_controls() {

        /** Start section */
        $this->start_controls_section( 'style_price',
            [
                'label' => esc_html__( 'Price', 'selection' ),
                'tab' => Controls_Manager::TAB_STYLE,
                'condition' => [ 'general_source' => 'woocommerce' ]
            ] );

        /** Width. */
        $this->add_responsive_control(
            'price_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ '%', 'px' ],
                'range' => [
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ],
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 100,
                ],
                'separator' => 'after',
                'selectors'  => [
                    '{{WRAPPER}} .mdp-imager-box-price' => 'width: {{size}}{{unit}};',
                ],
            ]
        );

        $this->get_settings_style_group(
            'price',
            '.mdp-imager-box-price',
            [],
            [
                'margin' => true,
                'padding' => true,
                'color' => true,
                'typography' => true,
                'text-shadow' => true,
                'alignment-justify' => true,
                'background' => true,
                'border' => true,
                'border-radius' => true,
                'box-shadow' => true,
            ]
        );

        /** End section */
        $this->end_controls_section();

    }

    /**
     * Settings for the image style tab.
     */
    public function get_image_style_controls() {

        /** Start section */
        $this->start_controls_section( 'style_image',
            [
                'label' => esc_html__( 'Image', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE,
            ] );

        $this->add_control(
            'image_position',
            [
                'label'   => esc_html__( 'Image position', 'selection' ),
                'type'    => Controls_Manager::SELECT,
                'default' => 'center center',
                'options' => [
                    'center center' => esc_html__( 'Center Center', 'selection' ),
                    'center left' => esc_html__( 'Center Left', 'selection' ),
                    'center right' => esc_html__( 'Center Right', 'selection' ),
                    'top center' => esc_html__( 'Top Center', 'selection' ),
                    'top left' => esc_html__( 'Top Left', 'selection' ),
                    'top right' => esc_html__( 'Top Right', 'selection' ),
                    'bottom center' => esc_html__( 'Bottom Center', 'selection' ),
                    'bottom left' => esc_html__( 'Bottom Left', 'selection' ),
                    'bottom right' => esc_html__( 'Bottom Right', 'selection' ),
                ],
                'selectors'  => [
                    '{{WRAPPER}} .mdp-imager-image' => 'background-position: {{value}};',
                ],
            ]
        );

        $this->add_control(
            'image_repeat',
            [
                'label'   => esc_html__( 'Image repeat', 'selection' ),
                'type'    => Controls_Manager::SELECT,
                'default' => 'no-repeat',
                'options' => [
                    'no-repeat' => esc_html__( 'No Repeat', 'selection' ),
                    'repeat' => esc_html__( 'Repeat', 'selection' ),
                    'repeat-x' => esc_html__( 'Repeat X', 'selection' ),
                    'repeat-y' => esc_html__( 'Repeat Y', 'selection' ),
                ],
                'selectors'  => [
                    '{{WRAPPER}} .mdp-imager-image' => 'background-repeat: {{value}};',
                ],
            ]
        );

        $this->add_control(
            'image_size',
            [
                'label'   => esc_html__( 'Image size', 'selection' ),
                'type'    => Controls_Manager::SELECT,
                'default' => 'cover',
                'options' => [
                    'cover' => esc_html__( 'Cover', 'selection' ),
                    'contain' => esc_html__( 'Contain', 'selection' ),
                    'auto' => esc_html__( 'Auto', 'selection' ),
                    'custom' => esc_html__( 'Custom', 'selection' ),
                ],
                'selectors'  => [
                    '{{WRAPPER}} .mdp-imager-image' => 'background-size: {{value}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'image_custom_size',
            [
                'label' => esc_html__( 'Image Custom Size', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ '%', 'px', 'em', 'vm' ],
                'range' => [
                    '%' => [
                        'min' => 0,
                        'max' => 200,
                        'step' => 1,
                    ],
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 100,
                ],
                'selectors'  => [
                    '{{WRAPPER}} .mdp-imager-image' => 'background-size: {{size}}{{unit}};',
                ],
                'condition' => [ 'image_size' => 'custom' ]
            ]
        );

        /** Width. */
        $this->add_responsive_control(
            'image_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ '%', 'px' ],
                'range' => [
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ],
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 100,
                ],
                'selectors'  => [
                    '{{WRAPPER}} .mdp-imager-box-image' => 'width: {{size}}{{unit}};',
                ],
                'separator' => 'before'
            ]
        );

        $this->add_responsive_control(
            'image_height',
            [
                'label' => esc_html__( 'Height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ],
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 200,
                ],
                'separator' => 'after',
                'selectors'  => [
                    '{{WRAPPER}} .mdp-imager-image' => 'height: {{size}}{{unit}};',
                ],
            ]
        );

        $this->get_settings_style_group(
            'image',
            '.mdp-imager-box-image',
            [],
            [
                'margin' => true,
                'padding' => true,
                'background' => true,
                'border' => true,
                'border-radius' => true,
                'box-shadow' => true,
            ]
        );

        /** CSS filter. */
        $this->add_group_control(
            Group_Control_Css_Filter::get_type(),
            [
                'name'     => 'image_css_filter',
                'label'    => esc_html__( 'CSS filter', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-imager-image',
            ]
        );

        /** End section */
        $this->end_controls_section();

    }

    /**
     * Settings for the overlay style tab.
     */
    public function get_overlay_style_controls() {

        /** Start section */
        $this->start_controls_section( 'style_overlay',
            [
                'label' => esc_html__( 'Overlay', 'selection' ),
                'tab' => Controls_Manager::TAB_STYLE,
                'condition' => [ 'general_style' => 'overlay' ]
            ] );

        $this->get_settings_style_group(
            'overlay',
            '.mdp-imager-box-overlay',
            [],
            [
                'margin' => true,
                'padding' => true,
                'background' => true,
                'border' => true,
                'border-radius' => true,
            ]
        );

        /** CSS filter. */
        $this->add_group_control(
            Group_Control_Css_Filter::get_type(),
            [
                'name'     => 'overlay_css_filter',
                'label'    => esc_html__( 'CSS filter', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-imager-box-overlay',
            ]
        );

        /** End section */
        $this->end_controls_section();

    }

    /**
     * Settings for the category style tab.
     */
    public function get_category_style_controls() {

        /** Start section */
        $this->start_controls_section( 'style_category',
            [
                'label' => esc_html__( 'Category', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE,
            ] );

        /** Width. */
        $this->add_responsive_control(
            'category_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ '%', 'px' ],
                'range' => [
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ],
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 100,
                ],
                'separator' => 'after',
                'selectors'  => [
                    '{{WRAPPER}} .mdp-imager-box-category' => 'width: {{size}}{{unit}};',
                ],
            ]
        );

        $this->get_settings_style_group(
            'category',
            '.mdp-imager-box-category',
            [],
            [
                'margin' => true,
                'padding' => true,
                'color' => true,
                'typography' => true,
                'text-shadow' => true,
                'alignment-justify' => true,
                'background' => true,
                'border' => true,
                'border-radius' => true,
                'box-shadow' => true,
            ]
        );

        /** End section */
        $this->end_controls_section();

    }

    /**
     * Settings for the date style tab.
     */
    public function get_date_style_controls(  ) {

        /** Start section */
        $this->start_controls_section( 'style_date',
            [
                'label' => esc_html__( 'Date', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE,
            ] );

        /** Width. */
        $this->add_responsive_control(
            'date_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ '%', 'px' ],
                'range' => [
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ],
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 100,
                ],
                'separator' => 'after',
                'selectors'  => [
                    '{{WRAPPER}} .mdp-imager-box-date' => 'width: {{size}}{{unit}};',
                ],
            ]
        );

        $this->get_settings_style_group(
            'date',
            '.mdp-imager-box-date',
            [],
            [
                'margin' => true,
                'padding' => true,
                'color' => true,
                'typography' => true,
                'text-shadow' => true,
                'alignment-justify' => true,
                'background' => true,
                'border' => true,
                'border-radius' => true,
                'box-shadow' => true,
            ]
        );

        /** End section */
        $this->end_controls_section();

    }

    /**
     * Add the widget controls.
     *
     * @since 1.0.0
     * @access protected
     *
     * @return void with category names
     **/
    protected function register_controls() {

        /**
         *  TAB: Content.
         */

        /** General section. */
        $this->get_general_content_controls();

        /** Layout section. */
        $this->get_layout_content_controls();

        /** ImageBox section. */
        $this->get_image_box_content_controls();


        /**
         *  TAB: Style.
         */

        /** Styles for grid. */
        $this->get_grid_style_controls();

        /** Styles for title. */
        $this->get_title_style_controls();

        /** Styles for description. */
        $this->get_description_style_controls();

        /** Styles for price. */
        $this->get_price_style_controls();

        /** Styles for image. */
        $this->get_image_style_controls();

        /** Styles for overlay. */
        $this->get_overlay_style_controls();

        /** Styles for category. */
        $this->get_category_style_controls();

        /** Styles for date. */
        $this->get_date_style_controls();


    }

    /**
     * Displaying a custom image.
     *
     * @param $settings - User added data.
     */
    public function get_custom_image( $settings ) {

        if( ! isset($settings['imagebox_item_image']['url']) ) { return; }

        ?>
        <div class="mdp-imager-box-image ">
            <div style="background-image: url( <?php echo esc_url( $settings['imagebox_item_image']['url'] ); ?> );"
                 class="mdp-imager-image"></div>
        </div>
        <?php

    }

    /**
     * Displaying a custom title.
     *
     * @param $settings - User added data.
     * @param $options - Header settings.
     */
    public function get_custom_title( $settings,  $options ) {
        if( isset($settings['imagebox_item_title']) ):

	        $target     = $settings['imagebox_link_item']['is_external'] === 'on' ||
	                      $settings['imagebox_link_item']['is_external'] === true ? '_blank': '';
	        $nofollow   = $settings['imagebox_link_item']['nofollow'] ? 'nofollow': '';
            ?>

            <div class="mdp-imager-box-title">
                <?php if( 'title' ===  $options['general_link_type'] &&
                          'yes' === $settings['imagebox_show_link'] ) : ?>
                    <a href="<?php echo esc_url($settings['imagebox_link_item']['url']); ?>"
                       target="<?php echo esc_html($target); ?>"
                       rel="<?php echo esc_html($nofollow); ?>">
                <?php  endif; ?>

                    <?php echo esc_html($settings['imagebox_item_title']); ?>

                <?php if( 'title' ===  $options['general_link_type'] &&
                          'yes' === $settings['imagebox_show_link'] ) : ?>
                    </a>
                <?php  endif; ?>
            </div>

        <?php
        endif;
    }

    /**
     * Displaying a custom description.
     *
     * @param $settings - User added data.
     */
    public function get_custom_description( $settings ) {
        if( isset($settings['imagebox_item_description']) ):
            ?>

            <p class="mdp-imager-box-description">
                <?php echo esc_html($settings['imagebox_item_description']); ?>
            </p>

        <?php
        endif;
    }

    /**
     * Displaying a custom date.
     *
     * @param $settings - User added data.
     */
    public function get_custom_date( $settings ) {
        if( isset($settings['imagebox_item_date']) ):
            $wp_date_format = get_option( 'date_format' );
            ?>

            <p class="mdp-imager-box-date">
                <?php echo esc_html( date( $wp_date_format, strtotime($settings['imagebox_item_date']) ) ); ?>
            </p>

        <?php
        endif;
    }

    /**
     * Displaying a custom category.
     *
     * @param $settings - User added data.
     */
    public function get_custom_category( $settings ) {
        if( isset($settings['imagebox_item_category']) && 'yes' === $settings['imagebox_show_category'] ):
            ?>

            <p class="mdp-imager-box-category">
                <?php echo esc_html($settings['imagebox_item_category']); ?>
            </p>

        <?php
        endif;
    }

    /**
     *  Display elements from the data entered by the user.
     *
     * @param $settings - User entered data.
     * @param $LayoutSortable - An array of user-mapped layouts.
     * @param $options - Setting for displaying custom items.
     */
    public function get_custom_image_box( $settings, $LayoutSortable,  $options ) {

        /** We get the class depending on the selected animation type. */
        $animation3D_class = ('yes' === $options['general_show_animation']) ? 'mdp-imager-item-3d-' . $this->get_id() : '';

        ?>

        <div class="mdp-imager-wrap <?php echo esc_attr( $options['imager_hover'] ) ?> <?php echo esc_attr( $options['imager_details_hover'] ) ?>-<?php echo esc_attr( $options['imager_details_hover_direction'] ) ?>">

            <?php
            /** Makes an entire block link. */
            if( 'box' ===  $options['general_link_type'] &&
                'yes' === $settings['imagebox_show_link'] ) :
                $target     = $settings['imagebox_link_item']['is_external'] === 'on' ||
                              $settings['imagebox_link_item']['is_external'] === true ? '_blank': '';
                $nofollow   = $settings['imagebox_link_item']['nofollow'] ? 'nofollow': '';
            ?>
                <a href="<?php echo esc_url($settings['imagebox_link_item']['url']); ?>"
                   target="<?php echo esc_html($target); ?>"
                   rel="<?php echo esc_html($nofollow); ?>">
            <?php  endif; ?>

                <div class="mdp-imager-item mdp-imager-box-grid <?php echo esc_attr($animation3D_class); ?>">

                    <?php
                    /** Overlay style. */
                    if( 'overlay' === $options['general_style'] ):
                    $this->get_custom_image( $settings );
                    ?>
                    <div class="mdp-imager-box-overlay">
                        <?php
                        endif;

                        /** We list the layouts in the given order. */
                        foreach ($LayoutSortable as $val) {

                            switch ( $val['name'] ) {
                                case 'image':
                                    if( 'overlay' !== $options['general_style'] ){
                                        $this->get_custom_image( $settings );
                                    }
                                    break;
                                case 'title':
                                    $this->get_custom_title( $settings,  $options );
                                    break;
                                case 'description':
                                    $this->get_custom_description( $settings );
                                    break;
                                case 'date':
                                    $this->get_custom_date( $settings );
                                    break;
                                case 'category':
                                    $this->get_custom_category( $settings );
                                    break;
                                case 'price':
                                    echo '';
                                    break;
                                default:
                                    echo esc_html__('You have not added any items to the layout.', 'selection');
                            }

                        }

                        ?>

                        <?php
                        /** End overlay style. */
                        if( 'overlay' === $options['general_style'] ): ?>
                    </div>
                <?php  endif; ?>

                </div>

                <?php
                /** End box link. */
                if( 'box' ===  $options['general_link_type'] && 'yes' === $settings['imagebox_show_link']) : ?>
            </a>
        <?php  endif; ?>



        </div>

        <?php
    }

    /**
     * Displaying the post image.
     *
     * @param $settings - Post data.
     */
    public function get_posts_image( $settings ) {

        $image_url = get_the_post_thumbnail_url( $settings->ID, 'large' );

        if( empty($image_url) ){
            $image_url =  UnityPlugin::get_url() . '/images/placeholder.png';
        }
        ?>

        <div class="mdp-imager-box-image">
            <div style="background-image: url( <?php echo esc_url($image_url); ?> );" class="mdp-imager-image"></div>
        </div>

        <?php
    }

    /**
     * Displaying the post title.
     *
     * @param $settings - Post data.
     * @param $options - Header settings.
     */
    public function get_posts_title( $settings, $options ) {
        if( isset($settings->post_title) ):
            ?>

            <div class="mdp-imager-box-title">

                <?php
                /** Makes the title a link. */
                if( 'title' === $options['general_link_type'] ) : ?>
                <a href="<?php echo esc_url( $settings->guid ); ?>" target="_blank">
                    <?php  endif;

                    echo esc_html( $settings->post_title );

                    if( 'title' === $options['general_link_type'] ) : ?>
                </a>
            <?php  endif; ?>

            </div>

        <?php
        endif;
    }

    /**
     *  Displays description of the post.
     *
     * @param $settings - Post data.
     */
    public function get_posts_description( $settings ) {

        if( '' !== $settings->post_excerpt ):
            ?>

            <p class="mdp-imager-box-description">
                <?php echo esc_html( $settings->post_excerpt ); ?>
            </p>

        <?php
        endif;
    }

    /**
     * Displays date of the post.
     *
     * @param $settings - Post data.
     */
    public function get_posts_date( $settings ) {
        if( isset($settings->post_date) ):
            $wp_date_format = get_option( 'date_format' );
            ?>

            <p class="mdp-imager-box-date">
                <?php echo esc_html( date( $wp_date_format, strtotime($settings->post_date) ) ); ?>
            </p>

        <?php
        endif;
    }

    /**
     * Displays category of the post.
     *
     * @param $settings - Post data.
     */
    public function get_posts_category( $settings ) {
        $category = get_the_category( $settings->ID );
        $wc_cat = get_the_terms( $settings->ID, 'product_cat' );

        if( isset($category[0]->name) || isset($wc_cat[0]->name) ):
            ?>

            <p class="mdp-imager-box-category">
                <?php
                if( isset($category[0]->name) ){
                    echo esc_html($category[0]->name);
                }elseif( isset($wc_cat[0]->name) ){
                    echo esc_html( $wc_cat[0]->name );
                }
                ?>
            </p>

        <?php
        endif;
    }

    /**
     * Displays price of the product.
     *
     * @param $settings - Post data.
     * @param $options - Settings for displaying product prices.
     */
    public function get_posts_price( $settings, $options ){

        $product = wc_get_product($settings->ID );
        if( 'woocommerce' === $options['general_source'] ):
            ?>

            <p class="mdp-imager-box-price">
                <?php echo $product->get_price_html(); ?>
            </p>

        <?php
        endif;
    }

    /**
     * Displaying items from post and product categories.
     *
     * @param $settings - User entered data.
     * @param $LayoutSortable - An array of user-mapped layouts.
     * @param $options - Setting for displaying custom items.
     */
    public function get_posts_image_box( $settings, $LayoutSortable, $options ){
        $animation3D_class = ('yes' === $options['general_show_animation']) ? 'mdp-imager-item-3d-'. $this->get_id() : '';

        ?>

        <div class="mdp-imager-wrap <?php echo esc_attr( $options['imager_hover'] ) ?> <?php echo esc_attr( $options['imager_details_hover'] ) ?>-<?php echo esc_attr( $options['imager_details_hover_direction'] ) ?>">

            <?php if( 'box' === $options['general_link_type'] ) : ?>
            <a href="<?php echo esc_url( $settings->guid ); ?>" target="_blank">
                <?php  endif; ?>

                <div class="mdp-imager-item mdp-imager-box-grid <?php echo esc_attr($animation3D_class); ?>">

                    <?php
                    if( 'overlay' === $options['general_style'] ):
                    $this->get_posts_image( $settings );
                    ?>

                    <div class="mdp-imager-box-overlay">

                        <?php
                        endif;

                        /** We list the layouts in the given order. */
                        foreach ($LayoutSortable as $val) {

                            switch ( $val['name'] ) {
                                case 'image':
                                    if( 'overlay' !== $options['general_style'] ) {
                                        $this->get_posts_image( $settings );
                                    }
                                    break;
                                case 'title':
                                    $this->get_posts_title( $settings, $options );
                                    break;
                                case 'description':
                                    $this->get_posts_description( $settings );
                                    break;
                                case 'date':
                                    $this->get_posts_date( $settings );
                                    break;
                                case 'category':
                                    $this->get_posts_category( $settings );
                                    break;
                                case 'price':
                                    $this->get_posts_price( $settings, $options );
                                    break;
                                default:
                                    echo esc_html__('You have not added any items to the layout.', 'selection');
                            }

                        }
                        ?>

                        <?php if( 'overlay' === $options['general_style'] ): ?>
                    </div>
                <?php  endif; ?>

                </div>

                <?php if( 'box' === $options['general_link_type'] ) : ?>
            </a>
        <?php  endif; ?>

        </div>
        <?php

    }

    /**
     * We return an array with the names of the layouts.
     *
     * @param $settings - we get an array with all the data.
     *
     * @return array
     */
    private function get_LayoutSortable( $settings ) {

        /** We get all the data on the lists of layouts. */
        $layout = $settings['layout_priority'];

        /** An array in which the names of the layouts and their width will be stored. */
        $res = array();

        /** We write the names of the layouts and their width into an array. */
        foreach ($layout as $key => $val){

            $res[] = [ 'name' => $val['layout_item'] ];
        }

        return $res;

    }

    /**
     * Displays all custom item.
     *
     * @param $settings - Settings for displaying custom item.
     * @param $LayoutSortable - An array of user-mapped layouts.
     */
    public function get_custom_info( $settings, $LayoutSortable ) {

        /** Count the number. */
        $quantity_custom = count( $settings['imagebox_item'] ) - 1;

        /** We display imagebox item in a cycle. */
        for ( $i = 0; $i <= $quantity_custom; $i++) {
            $this->get_custom_image_box(
                $settings['imagebox_item'][$i],
                $LayoutSortable,
                $settings
            );
        }

    }

    /**
     * Displays all posts from a category.
     *
     * @param $settings - Settings for displaying posts.
     * @param $LayoutSortable - An array of user-mapped layouts.
     */
    public function get_posts_info( $settings, $LayoutSortable ) {

        /** We get an array with post data. */
        $posts = get_posts( [ 'category_name' => $settings['general_posts_category'] ] );

        /** We count the number of imagebox item in the category. */
        $quantity_posts = count( $posts ) - 1;

        if( $quantity_posts > 0 ) {

            /** We display imagebox item in a cycle. */
            for ( $i = 0; $i <= $quantity_posts; $i ++ ) {
                $this->get_posts_image_box( $posts[ $i ], $LayoutSortable, $settings );
            }

        }else{
            echo esc_html__('There are no posts in this category.', 'selection');
        }
    }

    /**
     *  Displays all products from a category.
     *
     * @param $settings - Settings for displaying products.
     * @param $LayoutSortable - An array of user-mapped layouts.
     */
    public function get_woocommerce_info( $settings, $LayoutSortable ) {

        /** We get an array with post data. */
        $product = get_posts( [ 'product_cat' => $settings['general_products_category'], 'post_type'   => 'product' ] );

        /** We count the number of imagebox item in the category. */
        $quantity_product = count( $product ) - 1;

        if( $quantity_product > 0 ){
            /** We display imagebox item in a cycle. */
            for ( $i = 0; $i <= $quantity_product; $i++) {
                $this->get_posts_image_box( $product[$i], $LayoutSortable, $settings );
            }
        }else{
            echo esc_html__('There are no products in this category.', 'selection');
        }

    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @since 1.0.0
     * @access protected
     **/
    protected function render() {

        /** Get all the values from the admin panel. */
        $settings = $this->get_settings_for_display();

        /** We return an array with the names of the layouts and their width. */
        $LayoutSortable = $this->get_LayoutSortable( $settings );
        ?>

        <div class="mdp-imager-container">

            <?php
            switch ($settings['general_source']){

                case 'custom':
                    $this->get_custom_info( $settings, $LayoutSortable );
                    break;

                case 'posts':
                    $this->get_posts_info( $settings, $LayoutSortable );
                    break;

                case 'woocommerce':
                    $this->get_woocommerce_info($settings, $LayoutSortable);
                    break;

                default:
                    echo "The source of information was not found.";

            }
            ?>

        </div>

        <?php if ( 'yes' === $settings['general_show_animation'] && Plugin::$instance->editor->is_edit_mode() ) : ?>

            <?php if( is_admin() ) { ?>
                <script>
                    document.addEventListener( "DOMContentLoaded", function () {
                        jQuery( '.mdp-imager-item-3d-<?php echo $this->get_id(); ?>' ).tilt({
                            maxTilt: <?php echo esc_attr($settings['general_max_tilt']['size']); ?>
                        });
                    } );

                    if ( document.readyState === "complete" ) {
                        jQuery( '.mdp-imager-item-3d-<?php echo $this->get_id(); ?>' ).tilt({
                            maxTilt: <?php echo esc_attr($settings['general_max_tilt']['size']); ?>
                        });
                    }
                </script>
            <?php } ?>

        <?php endif;

        if ( 'yes' === $settings['general_show_animation'] ) : ?>

            <?php if ( ! is_admin() ) { ?>
                <script>
                    document.addEventListener("DOMContentLoaded", function() {
                        jQuery( '.mdp-imager-item-3d-<?php echo $this->get_id(); ?>' ).tilt({
                            maxTilt: <?php echo esc_attr($settings['general_max_tilt']['size']); ?>
                        });
                    } );
                </script>
            <?php } ?>

        <?php
        endif;

    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @since 1.0.0
     * @access public
     **/
    public function get_custom_help_url() {
        return 'https://docs.merkulov.design/category/imager-elementor/';
    }

}
