<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Progresser for Elementor
 * Customizable progress bars for Elementor editor.
 * Exclusively on https://1.envato.market/progresser-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Icons_Manager;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Utils;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Line Progress Bar - Custom Elementor Widget.
 **/
class line_bar_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style(
         'mdp-selection-elementor-admin',
            UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
                [],
                UnityPlugin::get_version()
        );

        wp_register_style(
         'mdp-progresser-elementor',
            UnityPlugin::get_url() . 'css/progresser-elementor' . UnityPlugin::get_suffix() . '.css',
                [],
                UnityPlugin::get_version()
        );
        wp_register_script(
         'mdp-progresser-elementor',
            UnityPlugin::get_url() . 'js/progresser-elementor' . UnityPlugin::get_suffix() . '.js',
                [ 'jquery' ],
                UnityPlugin::get_version(),
       true
        );
        wp_register_script(
            'mdp-progressbar',
              UnityPlugin::get_url() . 'js/progressbar' . UnityPlugin::get_suffix() . '.js',
                  [ 'jquery' ],
                  UnityPlugin::get_version(),
         true
        );
    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-progresser-line-bar';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Line Progress Bar', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-line-bar-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Progresser' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-progresser-elementor', 'mdp-selection-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'mdp-progresser-elementor', 'mdp-progressbar' ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> Example Content Section. */
        $this->section_content_progress_bar();

    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Style Progress Bar. */
        $this->section_style_progress_bar();

        /** Style -> Section Style Title. */
        $this->section_style_title();

        /** Style -> Section Style Number. */
        $this->section_style_number();

        /** Style -> Section Style Inner Text. */
        $this->section_style_inner_text();

    }

    /**
     * Add widget controls: Content -> Example Content Progress Bar.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_progress_bar() {

        $this->start_controls_section( 'section_content_progress_bar', [
            'label' => esc_html__( 'Progress Bar', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'progress_title',
            [
                'label' => esc_html__( 'Title', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'label_block' => true,
                'placeholder' => esc_html__( 'Type title here', 'selection' ),
                'default' => esc_html__( 'Progress Bar', 'selection' ),
                'dynamic' => [
                    'active' => true,
                ],
            ]
        );

        $this->add_control(
            'show_title_icon',
            [
                'label' => esc_html__( 'Show title icon', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'title_icon',
            [
                'label' => esc_html__( 'Title icon', 'selection' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-star',
                    'library' => 'solid',
                ],
                'condition' => [
                    'show_title_icon' => 'yes',
                ]
            ]
        );

        $this->add_control(
            'title_icon_position',
            [
                'label' => esc_html__( 'Title icon position', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'before',
                'options' => [
                    'before' => esc_html__( 'Before', 'selection' ),
                    'after' => esc_html__( 'After', 'selection' ),
                    'above-left' => esc_html__( 'Above left', 'selection' ),
                    'above-center' => esc_html__( 'Above center', 'selection' ),
                    'above-right' => esc_html__( 'Above right', 'selection' ),
                    'under-left' => esc_html__( 'Under left', 'selection' ),
                    'under-center' => esc_html__( 'Under center', 'selection' ),
                    'under-right' => esc_html__( 'Under right', 'selection' ),
                ],
                'condition' => [
                    'show_title_icon' => 'yes',
                ]
            ]
        );

        $this->add_control(
            'reflect_counter_title',
            [
                'label' => esc_html__( 'Reflect title and counter value', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'no',
                'separator' => 'before'
            ]
        );

        $this->add_control(
            'title_count_value_alignment',
            [
                'label' => esc_html__( 'Title alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                    'space-around' => [
                        'title' => esc_html__( 'Space around', 'selection' ),
                        'icon' => 'fa fa-compress-arrows-alt',
                    ],
                    'space-between' => [
                        'title' => esc_html__( 'Space between', 'selection' ),
                        'icon' => 'fa fa-expand-arrows-alt',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-progresser-elementor-text-container' => 'justify-content: {{VALUE}}',
                ],
                'default' => 'space-between',
                'toggle' => true,
            ]
        );


        $this->add_control(
            'title_position',
            [
                'label' => esc_html__( 'Title position', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'above-bar',
                'options' => [
                    'above-bar' => esc_html__( 'Above bar', 'selection' ),
                    'under-bar' => esc_html__( 'Under bar', 'selection' ),
                ],
            ]
        );


        $this->add_control(
            'inner_text',
            [
                'label' => esc_html__( 'Inner text', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'label_block' => true,
                'placeholder' => esc_html__( 'Type your inner text here', 'selection' ),
                'separator' => 'before',
                'dynamic' => [
                    'active' => true,
                ],
            ]
        );

        $this->add_control(
            'inner_text_alignment',
            [
                'label' => esc_html__( 'Inner text alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );


        $this->add_control(
            'separate_title',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );


        $this->add_control(
            'show_counter_value',
            [
                'label' => esc_html__( 'Display counter value', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'counter_value_type',
            [
                'label' => esc_html__( 'Counter value type', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'percents',
                'options' => [
                    'percents'  => esc_html__( 'Percents', 'plugin-elementor' ),
                    'custom-value' => esc_html__( 'Custom value', 'plugin-elementor' ),
                ],
            ]
        );

        $this->add_control(
            'counter_value',
            [
                'label' => esc_html__( 'Counter value', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 100,
                ],
                'dynamic' => [
                    'active' => true,
                ],
                'condition' => [
                    'counter_value_type' => 'percents'
                ]
            ]
        );

        $this->add_control(
            'custom_counter_value',
            [
                'label' => esc_html__( 'Counter value', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 1000,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 500,
                ],
                'dynamic' => [
                    'active' => true,
                ],
                'condition' => [
                    'counter_value_type' => 'custom-value'
                ]
            ]
        );

        $this->add_control(
            'max_custom_counter_value',
            [
                'label' => esc_html__( 'Max counter value', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 1000,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 500,
                ],
                'dynamic' => [
                    'active' => true,
                ],
                'condition' => [
                    'counter_value_type' => 'custom-value'
                ]
            ]
        );

        $this->add_control(
            'postfix_prefix_symbol',
            [
                'label' => esc_html__( 'Postfix/Prefix symbol', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( '%', 'selection' ),
                'dynamic' => [
                    'active' => true,
                ],
                'condition' => [
                    'show_counter_value' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'symbol_position',
            [
                'label' => esc_html__( 'Symbol position', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'after-number',
                'options' => [
                    'after-number' => esc_html__( 'After number', 'selection' ),
                    'before-number' => esc_html__( 'Before number', 'selection' ),
                ],
                'condition' => [
                    'show_counter_value' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'separate_symbol',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_control(
            'speed',
            [
                'label' => esc_html__( 'Speed', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 10000,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 3000,
                ],
            ]
        );

        $this->add_control(
            'animation_easing',
            [
                'label' => esc_html__( 'Easing', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'linear',
                'options' => [
                    'linear' => esc_html__( 'Linear', 'selection' ),
                    'easeIn' => esc_html__( 'Ease in', 'selection' ),
                    'easeOut' => esc_html__( 'Ease out', 'selection' ),
                    'easeInOut' => esc_html__( 'Ease in out', 'selection' ),
                ],
            ]
        );

        $this->add_control(
            'start_animation',
            [
                'label' => esc_html__( 'Start animation', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'one-time',
                'options' => [
                    'one-time' => esc_html__( 'Once', 'selection' ),
                    'on-scroll' => esc_html__( 'On scroll', 'selection' ),
                    'in-viewport' => esc_html__( 'In viewport', 'selection' ),
                ],
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Function for generating margin padding controls.
     *
     * @param $section_id
     * @param $html_class
     * @return void
     * @since 1.0.0
     * @access private
     *
     */
    private function generate_margin_padding_controls( $section_id, $html_class ) {
        $this->add_responsive_control(
            $section_id.'_margin',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'separator' => 'after',
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );
    }


    /**
     * Function for generating typography and tabs controls.
     *
     * @param $section_id
     * @param $opts
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_typography_tabs_controls($section_id, $opts = [] ) {
        $style_opts = [
            'html_class' => array_key_exists( 'html_class', $opts ) ? $opts['html_class'] : '',
            'include_color' => array_key_exists( 'include_color', $opts ) ? $opts['include_color'] : true,
            'include_bg' => array_key_exists( 'include_bg', $opts ) ? $opts['include_color'] : true,
            'include_typography' => array_key_exists( 'include_typography', $opts ) ? $opts['include_typography'] : true,
            'include_transition' => array_key_exists( 'include_transition', $opts ) ? $opts['include_transition'] : true,
            'additional_color' => array_key_exists( 'additional_color', $opts ) ? $opts['additional_color'] : false,
            'color_class' => array_key_exists( 'color_class', $opts ) ? $opts['color_class'] : '',
            'color_prefix' => array_key_exists( 'color_prefix', $opts ) ? $opts['color_prefix'] : '',
            'color_hover_class' => array_key_exists( 'color_hover_class', $opts ) ? $opts['color_hover_class'] : '',
            'color_hover_additional_class' => array_key_exists( 'color_hover_additional_class', $opts ) ? $opts['color_hover_additional_class'] : '',
            'color_additional_class' => array_key_exists( 'color_additional_class', $opts ) ? $opts['color_additional_class'] : '',
            'color_value' => array_key_exists( 'color_value', $opts ) ? $opts['color_value'] : '',
            'color_hover_selector' => array_key_exists( 'color_hover_selector', $opts ) ? $opts['color_hover_selector'] : '',
            'additional_color_name' => array_key_exists( 'additional_color_name', $opts ) ? $opts['additional_color_name'] : '',
            'additional_color_class' => array_key_exists( 'additional_color_class', $opts ) ? $opts['additional_color_class'] : '',
            'additional_color_additional_class' => array_key_exists( 'additional_color_additional_class', $opts ) ? $opts['additional_color_additional_class'] : '',
            'additional_color_hover_class' => array_key_exists( 'additional_color_hover_class', $opts ) ? $opts['additional_color_hover_class'] : '',
            'additional_color_hover_additional_class' => array_key_exists( 'additional_color_hover_additional_class', $opts ) ? $opts['additional_color_hover_additional_class'] : '',
            'additional_transition_selector' => array_key_exists( 'additional_transition_selector', $opts ) ? $opts['additional_transition_selector'] : '',
            'typography_class' => array_key_exists( 'typography_class', $opts ) ? $opts['typography_class'] : '',
            'color_scheme_default' => array_key_exists( 'color_scheme_default', $opts ) ? $opts['color_scheme_default'] : Color::COLOR_3,
            'additional_color_scheme_default' => array_key_exists( 'additional_color_scheme_default', $opts ) ? $opts['additional_color_scheme_default'] : Color::COLOR_3,
            'additional_html_selector' => array_key_exists( 'additional_html_selector', $opts ) ? $opts['additional_html_selector'] : '',
            'additional_html_selector_hover' => array_key_exists( 'additional_html_selector_hover', $opts ) ? $opts['additional_html_selector_hover'] : ''
        ];


        if ( $style_opts['include_typography'] ) {
            $this->add_group_control(
                Group_Control_Typography::get_type(),
                [
                    'name' => $section_id . '_typography',
                    'label' => esc_html__('Typography', 'selection'),
                    'scheme' => Typography::TYPOGRAPHY_1,
                    'selector' => "{{WRAPPER}} .".$style_opts['typography_class'],
                ]
            );
        }

        $this->start_controls_tabs( $section_id.'_style_tabs' );

        $this->start_controls_tab( $section_id.'_normal_style_tab', ['label' => esc_html__( 'NORMAL', 'selection' )] );

        if ( $style_opts['include_color'] ) {

            $this->add_control(
                $section_id . '_normal_text_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_class'] => 'color: {{VALUE}} !important;',
                        "{{WRAPPER}} .".$style_opts['color_additional_class'] => $style_opts['color_value'],
                    ],
                ]
            );

        }

        if ( $style_opts['additional_color'] ) {
            $this->add_control(
                $section_id . '_' . $style_opts['additional_color_name'] . '_normal_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_color_name'], 'selection' ),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_color_class'] => 'color: {{VALUE}} !important;',
                        "{{WRAPPER}} .".$style_opts['additional_color_additional_class'] => $style_opts['color_value'],

                    ],
                ]
            );
        }

        if ( $style_opts['include_bg'] ) {

            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_normal_background',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
                ]
            );

        }

        $this->add_control(
            $section_id . '_separate_normal',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_normal',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'].",{{WRAPPER}} .".$style_opts['additional_html_selector'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab( $section_id.'_hover_style_tab', ['label' => esc_html__( 'HOVER', 'selection' )] );

        if ( $style_opts['include_color'] ) {
            $this->add_control(
                $section_id . '_hover_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_hover_class'] => 'color: {{VALUE}} !important;',
                        "{{WRAPPER}} .".$style_opts['color_hover_additional_class'] => $style_opts['color_value'],
                    ],
                ]
            );
        }

        if ( $style_opts['additional_color'] ) {
            $this->add_control(
                $section_id . '_' . $style_opts['additional_color_name'] . '_hover_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_color_name'], 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_color_hover_class'] => 'color: {{VALUE}} !important;',
                        "{{WRAPPER}} .".$style_opts['additional_color_hover_additional_class'] => $style_opts['color_value'],
                    ],
                ]
            );
        }

        if ( $style_opts['include_bg'] ) {
            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_background_hover',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover",
                ]
            );
        }

        if ( $style_opts['include_transition'] ) {
            $this->add_control(
                $section_id.'_hover_transition',
                [
                    'label' => esc_html__( 'Hover transition duration', 'selection' ),
                    'type' => Controls_Manager::SLIDER,
                    'size_units' => [ 's' ],
                    'range' => [
                        's' => [
                            'min' => 0.1,
                            'max' => 5,
                            'step' => 0.1,
                        ],
                    ],
                    'default' => [
                        'unit' => 's',
                        'size' => 0,
                    ],
                    'selectors' => [
                        '{{WRAPPER}} .'.$style_opts['html_class'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};',
                        $style_opts['additional_transition_selector'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};;'
                    ],
                ]
            );
        }

        $this->add_control(
            $section_id.'_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_hover',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover",
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'].":hover, {{WRAPPER}} .".$style_opts['additional_html_selector_hover'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover",
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();
    }

    /**
     * Add widget controls: Style -> Section Style Progress Bar.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_progress_bar() {

        $this->start_controls_section( 'section_style_progress_bar', [
            'label' => esc_html__( 'Progress Bar', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls( 'section_style_progress_bar', 'mdp-progresser-bar-container' );

        $this->add_control(
            'progress_bar_height',
            [
                'label' => esc_html__( 'Progress line height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 5
                ],
            ]
        );

        $this->add_responsive_control(
            'progress_bar_width',
            [
                'label' => esc_html__( 'Progress line width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 100,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-progresser-bar-container svg' => 'width: {{SIZE}}{{UNIT}} !important'
                ]
            ]
        );

        $this->add_control(
            'rounded_stroke_linecap',
            [
                'label' => esc_html__( 'Rounded stroke linecap', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $this->add_control(
            'linecap_divider',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $style_options = [
                'html_class' => 'mdp-progresser-bar-container',
                'include_transition' => true,
                'additional_color' => true,
                'include_typography' => false,
                'color_additional_class' => 'mdp-progresser-bar-container path:last-child',
                'additional_color_additional_class' => 'mdp-progresser-bar-container path:first-child',
                'color_value' => 'stroke: {{VALUE}} !important;',
                'additional_color_name' => 'Empty Line Color',
                'additional_color_hover_additional_class' => 'mdp-progresser-bar-container:hover path:first-child',
                'additional_transition_selector' => '{{WRAPPER}} .mdp-progresser-bar-container path:last-child, {{WRAPPER}} .mdp-progresser-bar-container path:first-child',
                'color_hover_additional_class' => 'mdp-progresser-bar-container:hover path:last-child',
                'color_prefix' => 'Line ',
                'color_scheme_default' => Color::COLOR_4,
                'additional_html_selector' => 'mdp-progresser-bar-container svg',
                'additional_html_selector_hover' => 'mdp-progresser-bar-container:hover svg'

        ];

        $this->generate_typography_tabs_controls( 'section_style_progress_bar',  $style_options );



        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Title.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_title()
    {

        $this->start_controls_section( 'section_style_title', [
            'label' => esc_html__('Title', 'selection'),
            'tab' => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls( 'section_style_title', 'mdp-progresser-elementor-title-container' );


        $this->add_responsive_control(
            'icon_spacing',
            [
                'label' => esc_html__( 'Icon spacing', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .mdp-progresser-elementor-icon" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );


        $this->add_responsive_control(
            'title_icon_size',
            [
                'label' => esc_html__( 'Title icon size', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 20,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-progresser-elementor-icon' => 'font-size: {{SIZE}}{{UNIT}}; width: {{SIZE}}{{UNIT}}; height: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'separate_icon_size',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $style_options = [
            'html_class' => 'mdp-progresser-elementor-title-container',
            'color_class' => 'mdp-progresser-elementor-title',
            'color_hover_class' => 'mdp-progresser-elementor-title-container:hover .mdp-progresser-elementor-title',
            'typography_class' => 'mdp-progresser-elementor-title',
            'additional_color' => true,
            'additional_color_class' => 'mdp-progresser-elementor-icon',
            'additional_color_hover_class' => 'mdp-progresser-elementor-title-container:hover .mdp-progresser-elementor-icon',
            'additional_color_name' => 'Icon Color',
            'additional_transition_selector' => '{{WRAPPER}} .mdp-progresser-elementor-title, {{WRAPPER}} .mdp-progresser-elementor-icon'


        ];

        $this->generate_typography_tabs_controls( 'section_style_title', $style_options );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Number.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_number()
    {

        $this->start_controls_section('section_style_number', [
            'label' => esc_html__('Number', 'selection'),
            'tab' => Controls_Manager::TAB_STYLE,
            'condition' => [
                'show_counter_value' => 'yes'
            ]
        ]);

        $this->generate_margin_padding_controls( 'section_style_number', 'mdp-progress-elementor-count-value' );


        $style_options = [
            'html_class' => 'mdp-progress-elementor-count-value',
            'color_class' => 'mdp-progress-elementor-count-value',
            'color_hover_class' => 'mdp-progress-elementor-count-value:hover .mdp-progress-elementor-count-number, {{WRAPPER}} .mdp-progress-elementor-count-value:hover .mdp-progress-elementor-count-symbol',
            'typography_class' => 'mdp-progress-elementor-count-value .mdp-progress-elementor-count-number, {{WRAPPER}} .mdp-progress-elementor-count-value .mdp-progress-elementor-count-symbol',
            'additional_transition_selector' => '{{WRAPPER}} .mdp-progress-elementor-count-number, {{WRAPPER}} .mdp-progress-elementor-count-symbol'

        ];

        $this->generate_typography_tabs_controls( 'section_style_number', $style_options );


        $this->end_controls_section();
    }

    /**
     * Add widget controls: Style -> Section Style Inner Text.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_inner_text()
    {

        $this->start_controls_section('section_style_inner_text', [
            'label' => esc_html__('Inner text', 'selection'),
            'tab' => Controls_Manager::TAB_STYLE
        ]);

        $this->generate_margin_padding_controls( 'section_style_inner_text', 'mdp-progresser-elementor-inner-text' );


        $style_options = [
            'html_class' => 'mdp-progresser-elementor-inner-text',
            'color_class' => 'mdp-progresser-elementor-inner-text',
            'color_hover_class' => 'mdp-progresser-elementor-inner-text:hover',
            'typography_class' => 'mdp-progresser-elementor-inner-text',
        ];

        $this->generate_typography_tabs_controls( 'section_style_inner_text', $style_options );


        $this->end_controls_section();
    }


    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {
        $settings = $this->get_settings_for_display();

        ?>
        <!-- Start Progresser for Elementor WordPress Plugin -->
        <div class="mdp-progresser-line-bar-elementor-box
            mdp-progresser-elementor-bar mdp-progresser-title-position-<?php esc_attr_e( $settings['title_position'] ); ?>
            mdp-progresser-line-bar-elementor-box-<?php esc_attr_e( $this->get_id() ); ?>"
            data-max-counter-value="<?php echo $settings['counter_value_type'] === 'custom-value' ?
                esc_attr( $settings['max_custom_counter_value']['size'] ) :
                esc_attr( 100 ) ?>"
            data-counter-value="<?php echo $settings['counter_value_type'] === 'custom-value' ?
                esc_attr( $settings['custom_counter_value']['size'] ) :
                esc_attr( $settings['counter_value']['size'] ); ?>"
            data-progress-bar-height="<?php echo esc_attr( $settings['progress_bar_height']['size'] ) ?>"
            data-inner-text-alignment="<?php echo esc_attr( $settings['inner_text_alignment'] ) ?>"
            data-inner-text="<?php echo esc_attr( $settings['inner_text'] ) ?>"
            data-animation-duration="<?php echo esc_attr( $settings['speed']['size'] ) ?>"
            data-animation-easing="<?php echo esc_attr( $settings['animation_easing'] ) ?>"
            data-start-animation="<?php echo esc_attr( $settings['start_animation'] ); ?>"

        >
         <div class="mdp-progresser-elementor-text-container <?php if ( $settings['reflect_counter_title'] === 'yes' ): ?>mdp-progresser-elementor-text-container-reflect<?php endif; ?>">
             <div class="mdp-progresser-elementor-title-container <?php if ( $settings['show_title_icon'] === 'yes' ): ?>mdp-progresser-elementor-title-icon-position-<?php esc_attr_e( $settings['title_icon_position'] ); ?><?php endif; ?>">
                 <?php if ( $settings['show_title_icon'] === 'yes' ): ?>
                     <div class="mdp-progresser-elementor-icon">
                         <?php Icons_Manager::render_icon( $settings['title_icon'], ['aria-hidden' => 'true'] ); ?>
                     </div>
                 <?php endif; ?>
                 <div class="mdp-progresser-elementor-title">
                     <?php esc_html_e( $settings['progress_title'] ); ?>
                 </div>
             </div>
             <?php if ( $settings['show_counter_value'] === 'yes' ): ?>
             <div class="mdp-progress-elementor-count-value mdp-progress-count-symbol-position-<?php esc_attr_e( $settings['symbol_position'] ); ?>">
                 <div class="mdp-progress-elementor-count-number"><?php $settings['counter_value_type'] === 'custom-value' ?
                         esc_attr_e( $settings['custom_counter_value']['size'] ) :
                         esc_attr_e( $settings["counter_value"]["size"] ); ?></div>
                 <div class="mdp-progress-elementor-count-symbol"><?php esc_attr_e( $settings['postfix_prefix_symbol'] ); ?></div>
             </div>
             <?php endif; ?>
         </div>
         <div class="mdp-progresser-bar-container <?php if ( $settings['rounded_stroke_linecap'] === 'yes' ): ?>mdp-progresser-elementor-line-rounded<?php endif; ?> mdp-progresser-bar-container-<?php esc_attr_e( $this->get_id() ); ?>"></div>
        </div>
        <!-- End Progresser for Elementor WordPress Plugin -->


	    <?php
    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/progresser';

    }

}
