<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Liner for Elementor
 * The plugin for Elementor which allows you to animate and manage SVG images. Customize each SVG path to create a unique style of your animation.
 * Exclusively on https://1.envato.market/liner-elementor
 *
 * @encoding        UTF-8
 * @version         2.0.0
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Css_Filter;
use Elementor\Repeater;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Core\Schemes\Color;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Liner - Custom Elementor Widget.
 **/
class liner_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style(
         'mdp-selection-elementor-admin',
            UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
                [],
                UnityPlugin::get_version()
        );
        wp_register_style(
        'mdp-liner-elementor',
           UnityPlugin::get_url() . 'css/liner-elementor' . UnityPlugin::get_suffix() . '.css',
               [],
               UnityPlugin::get_version()
        );

	    wp_register_script(
        'mdp-liner-elementor',
           UnityPlugin::get_url() . 'js/liner-elementor' . UnityPlugin::get_suffix() . '.js',
               [ 'jquery' ],
               UnityPlugin::get_version(),
      true
        );
	    wp_register_script(
        'mdp-vivus-js',
           UnityPlugin::get_url() . 'js/vivus' . UnityPlugin::get_suffix() . '.js',
               [ 'jquery' ],
               UnityPlugin::get_version(),
      true
        );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-liner-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Animated Icon', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-liner-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Animated Icon' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-liner-elementor', 'mdp-selection-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'mdp-vivus-js', 'mdp-liner-elementor' ];

    }

    /**
     * Section Properties.
     */
    public function get_properties_content_controls(){

        /** Start section */
        $this->start_controls_section(
            'properties_content',
            [
                'label' => esc_html__( 'Properties', 'selection' ),
                'tab' => Controls_Manager::TAB_CONTENT,
            ]
        );

        /** Image. */
        $this->add_control(
            'animation_content__image',
            [
                'label' =>esc_html__( 'Select SVG', 'avatar-elementor' ),
                'type' => Controls_Manager::MEDIA,
                'default' => [
                    'url' => UnityPlugin::get_url() . 'images/m.svg',
                ],
            ]
        );

        /** Type animation  */
        $this->add_control(
            'properties_type_animation',
            [
                'label' => esc_html__( 'Type animation', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'oneByOne',
                'options' => [
                    'oneByOne' => esc_html__( 'One By One', 'selection' ),
                    'delayed'  => esc_html__( 'Delayed', 'selection' ),
                    'sync' => esc_html__( 'Sync', 'selection' ),
                ],
            ]
        );

        /** Delayed description. */
        $this->add_control(
            'properties_raw_delayed',
            [
                'type' => Controls_Manager::RAW_HTML,
                'raw' => '<div class="elementor-descriptor">' . esc_html__(
                        'Every path element is drawn at the same time with a small delay at the start.',
                        'selection'
                    ) . '</div>',
                'conditions' => [
                    'terms' => [
                        [
                            'name' => 'properties_type_animation',
                            'operator' => '===',
                            'value' => 'delayed'
                        ]
                    ]
                ]
            ]
        );

        /** Sync description. */
        $this->add_control(
            'properties_raw_sync',
            [
                'type' => Controls_Manager::RAW_HTML,
                'raw' => '<div class="elementor-descriptor">' . esc_html__(
                        'Each line is drawn synchronously. They all start and finish at the same time.',
                        'selection'
                    ) . '</div>',
                'conditions' => [
                    'terms' => [
                        [
                            'name' => 'properties_type_animation',
                            'operator' => '===',
                            'value' => 'sync'
                        ]
                    ]
                ]
            ]
        );

        /** One By One description. */
        $this->add_control(
            'properties_raw_oneByOne',
            [
                'type' => Controls_Manager::RAW_HTML,
                'raw' => '<div class="elementor-descriptor">' . esc_html__(
                        'Each path element is drawn one after the other.',
                        'selection'
                    ) . '</div>',
                'conditions' => [
                    'terms' => [
                        [
                            'name' => 'properties_type_animation',
                            'operator' => '===',
                            'value' => 'oneByOne'
                        ]
                    ]
                ]
            ]
        );

        /** Start animation  */
        $this->add_control(
            'properties_start_animation',
            [
                'label' => esc_html__( 'Start animation', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'inViewport',
                'options' => [
                    'inViewport'  => esc_html__( 'In viewport', 'selection' ),
                    'autostart' => esc_html__( 'Auto start', 'selection' ),
                    'hover' => esc_html__( 'Hover', 'selection' ),
                    'click' => esc_html__( 'Click', 'selection' ),
                ],
            ]
        );

        /** Animation Timing  */
        $this->add_control(
            'properties_animation_timing',
            [
                'label' => esc_html__( 'Animation Timing', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'EASE',
                'options' => [
                    'EASE'  => esc_html__( 'Ease', 'selection' ),
                    'EASE_IN' => esc_html__( 'Ease in', 'selection' ),
                    'EASE_OUT' => esc_html__( 'Ease out', 'selection' ),
                    'EASE_OUT_BOUNCE' => esc_html__( 'Ease out bounce', 'selection' ),
                ],
            ]
        );

        /** Path Timing  */
        $this->add_control(
            'properties_path_timing',
            [
                'label' => esc_html__( 'Path Timing', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'EASE',
                'options' => [
                    'EASE'  => esc_html__( 'Ease', 'selection' ),
                    'EASE_IN' => esc_html__( 'Ease in', 'selection' ),
                    'EASE_OUT' => esc_html__( 'Ease out', 'selection' ),
                    'EASE_OUT_BOUNCE' => esc_html__( 'Ease out bounce', 'selection' ),
                ],
            ]
        );

        /** Animation delay. */
        $this->add_responsive_control(
            'properties_animation_delay',
            [
                'label' => esc_html__( 'Animation delay (ms)', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'description' => esc_html__( 'Time between the drawing of first and last path, in frames.', 'selection' ),
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 10,
                ],
                'separator' => 'after',
                'condition' => ['properties_type_animation' => 'delayed'],
            ]
        );

        /** Animation duration. */
        $this->add_responsive_control(
            'properties_animation_duration',
            [
                'label' => esc_html__( 'Animation duration', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'description' => esc_html__( 'Animation duration, in frames.', 'selection' ),
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 2,
                        'max' => 1000,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 400,
                ],
                'separator' => 'after',
            ]
        );

        /** Reverse Stack */
        $this->add_control(
            'properties_reverse_stack',
            [
                'label' => esc_html__( 'Reverse Stack', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => '',
                'condition' => ['properties_type_animation' => ['oneByOne', 'scenario-sync']],
            ]
        );

        /** Dash Gap. */
        $this->add_responsive_control(
            'properties_dash_gap',
            [
                'label' => esc_html__( 'Dash Gap', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 2,
                ],
                'separator' => 'after',
                'description' => esc_html__( 'Whitespace extra margin between dashes. Increase it in case of glitches at the initial state of the animation.', 'selection' ),
            ]
        );

        /** Reverse Stack */
        $this->add_control(
            'loop',
            [
                'label' => esc_html__( 'Loop', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'no',
//                'condition' => ['properties_type_animation' => ['oneByOne', 'scenario-sync']],
            ]
        );

        /** Include link. */
        $this->add_control(
            'properties_show_link',
            [
                'label' => esc_html__( 'Include link', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
            ]
        );

        /** URL */
        $this->add_control(
            'properties_link',
            [
                'label' => esc_html__( 'URL', 'selection' ),
                'type' => Controls_Manager::URL,
                'show_external' => true,
                'default' => [
                    'url' => '',
                ],
                'condition' => ['properties_show_link' => 'yes'],
            ]
        );

        /** End section */
        $this->end_controls_section();

    }


    /**
     * Section Animation style
     */
    public function get_animation_style_controls() {

        /** Start section */
        $this->start_controls_section( 'style_animation',
            [
                'label' => esc_html__( 'Animation style', 'selection' ),
                'tab' => Controls_Manager::TAB_STYLE,
            ] );

        /** Size */
        $this->add_responsive_control(
            'animation_style_size',
            [
                'label' => esc_html__( 'Size', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ '%', 'px' ],
                'range' => [
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ],
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 100,
                ],
                'separator' => 'after',
                'selectors'  => [
                    '{{WRAPPER}} .mdp-liner-box' => 'width: {{size}}{{unit}};',
                ],
            ]
        );

        /** Color */
        $this->add_control(
            'animation_style_color',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_1,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-liner-box svg *' => 'stroke: {{VALUE}}',
                ],
            ]
        );

        /** Line Width */
        $this->add_responsive_control(
            'animation_style_line_width',
            [
                'label' => esc_html__( 'Line Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 1,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-liner-box svg *' => 'stroke-width: {{SIZE}}{{UNIT}}',
                ],
            ]
        );

        /** Rotate */
        $this->add_responsive_control(
            'animation_style_rotate',
            [
                'label' => esc_html__( 'Rotate', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'deg' ],
                'range' => [
                    'deg' => [
                        'min' => -360,
                        'max' => 360,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'deg',
                    'size' => 0,
                ],
                'selectors' => [
                    '(desktop){{WRAPPER}} .mdp-liner-box' => 'transform: translate({{animation_style_offset_x.SIZE}}{{animation_style_offset_x.UNIT}}, {{animation_style_offset_y.SIZE}}{{animation_style_offset_y.UNIT}}) rotate({{animation_style_rotate.SIZE}}deg);',
                    '(tablet){{WRAPPER}} .mdp-liner-box' => 'transform: translate({{animation_style_offset_x_tablet.SIZE}}{{animation_style_offset_x.UNIT}}, {{animation_style_offset_y_tablet.SIZE}}{{animation_style_offset_y.UNIT}}) rotate({{animation_style_rotate.SIZE}}deg);',
                    '(mobile){{WRAPPER}} .mdp-liner-box' => 'transform: translate({{animation_style_offset_x_mobile.SIZE}}{{animation_style_offset_x.UNIT}}, {{animation_style_offset_y_mobile.SIZE}}{{animation_style_offset_y.UNIT}}) rotate({{animation_style_rotate.SIZE}}deg);',
                ],
            ]
        );

        /** Offset-X */
        $this->add_responsive_control(
            'animation_style_offset_x',
            [
                'label' => esc_html__( 'Offset-X', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => -500,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 0,
                ],
                'selectors' => [
                    '(desktop){{WRAPPER}} .mdp-liner-box' => 'transform: translate({{animation_style_offset_x.SIZE}}{{animation_style_offset_x.UNIT}}, {{animation_style_offset_y.SIZE}}{{animation_style_offset_y.UNIT}}) rotate({{animation_style_rotate.SIZE}}deg);',
                    '(tablet){{WRAPPER}} .mdp-liner-box' => 'transform: translate({{animation_style_offset_x_tablet.SIZE}}{{animation_style_offset_x.UNIT}}, {{animation_style_offset_y_tablet.SIZE}}{{animation_style_offset_y.UNIT}}) rotate({{animation_style_rotate.SIZE}}deg);',
                    '(mobile){{WRAPPER}} .mdp-liner-box' => 'transform: translate({{animation_style_offset_x_mobile.SIZE}}{{animation_style_offset_x.UNIT}}, {{animation_style_offset_y_mobile.SIZE}}{{animation_style_offset_y.UNIT}}) rotate({{animation_style_rotate.SIZE}}deg);',
                ],
            ]
        );

        /** Offset-Y */
        $this->add_responsive_control(
            'animation_style_offset_y',
            [
                'label' => esc_html__( 'Offset-Y', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => -500,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 0,
                ],
                'selectors' => [
                    '(desktop){{WRAPPER}} .mdp-liner-box' => 'transform: translate({{animation_style_offset_x.SIZE}}{{animation_style_offset_x.UNIT}}, {{animation_style_offset_y.SIZE}}{{animation_style_offset_y.UNIT}}) rotate({{animation_style_rotate.SIZE}}deg);',
                    '(tablet){{WRAPPER}} .mdp-liner-box' => 'transform: translate({{animation_style_offset_x_tablet.SIZE}}{{animation_style_offset_x.UNIT}}, {{animation_style_offset_y_tablet.SIZE}}{{animation_style_offset_y.UNIT}}) rotate({{animation_style_rotate.SIZE}}deg);',
                    '(mobile){{WRAPPER}} .mdp-liner-box' => 'transform: translate({{animation_style_offset_x_mobile.SIZE}}{{animation_style_offset_x.UNIT}}, {{animation_style_offset_y_mobile.SIZE}}{{animation_style_offset_y.UNIT}}) rotate({{animation_style_rotate.SIZE}}deg);',
                ],
            ]
        );

        /** Linecap */
        $this->add_control(
            'animation_style_linecap',
            [
                'label' => esc_html__( 'Linecap', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'butt',
                'separator' => 'after',
                'options' => [
                    'butt'  => esc_html__( 'Butt', 'selection' ),
                    'round' => esc_html__( 'Round', 'selection' ),
                    'square' => esc_html__( 'Square', 'selection' ),
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-liner-box svg *' => 'stroke-linecap: {{VALUE}}',
                ],
            ]
        );

        /** Box Shadow */
        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'animation_style_shadow',
                'label' => __( 'Box Shadow', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-liner-box',
            ]
        );

        /** CSS filter. */
        $this->add_group_control(
            Group_Control_Css_Filter::get_type(),
            [
                'name'     => 'animation_style_css_filter',
                'label'    => esc_html__( 'CSS filter', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-liner-box',
            ]
        );

        /** End section */
        $this->end_controls_section();

    }

    /**
     * Section Path styles
     */
    public function get_path_style_controls() {

        /** Start section */
        $this->start_controls_section( 'style_path',
            [
                'label' => esc_html__( 'Path styles', 'selection' ),
                'tab' => Controls_Manager::TAB_STYLE,
            ] );

        $path = new Repeater();

        /** Path number. */
        $path->add_control(
            'path_number',
            [
                'label' => esc_html__( 'Path number', 'plugin-domain' ),
                'type' => Controls_Manager::NUMBER,
                'min' => 1,
                'step' => 1,
                'default' => 1,
            ]
        );

        /** Color */
        $path->add_control(
            'path_style_color',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_1,
                ],
            ]
        );

        /** Line Width */
        $path->add_control(
            'path_style_line_width',
            [
                'label' => esc_html__( 'Line Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 1,
                ],
            ]
        );

        /** Offset-X */
        $path->add_control(
            'path_style_offset_x',
            [
                'label' => esc_html__( 'Offset-X', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => -500,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 0,
                ],
            ]
        );

        /** Offset-Y */
        $path->add_control(
            'path_style_offset_y',
            [
                'label' => esc_html__( 'Offset-Y', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => -500,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 0,
                ],
            ]
        );

        /** Rotate */
        $path->add_control(
            'path_style_rotate',
            [
                'label' => esc_html__( 'Rotate', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => -360,
                        'max' => 360,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 0,
                ],
            ]
        );

        $this->add_control(
            'path_priority',
            [
                'label'       => esc_html__( 'Paths', 'selection' ),
                'type'        => Controls_Manager::REPEATER,
                'fields'      => $path->get_controls(),
                'default'     => [],
                'separator'   => 'before',
                'title_field' => 'Paths'
            ]
        );

        /** End section */
        $this->end_controls_section();

    }

    /**
     * Add the widget controls.
     *
     * @since 1.0.0
     * @access protected
     *
     * @return void with category names
     **/
    protected function register_controls() {

        /**
         *  TAB: Content.
         */

        /** Properties. */
        $this->get_properties_content_controls();

        /**
         *  TAB: Style.
         */

        /** Animation */
        $this->get_animation_style_controls();

        /** Path */
        $this->get_path_style_controls();

    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @since 1.0.0
     * @access protected
     **/
    protected function render() {

        /** Get all the values from the admin panel. */
        $settings = $this->get_settings_for_display();
        $idSection = $this->get_id();
        $target = '';
        $nofollow = '';
        $delayed = '';

        if ( is_array( $settings['properties_link'] ) ) {

            $target = $settings['properties_link']['is_external'] ? ' target="_blank"' : '';
            $nofollow = $settings['properties_link']['nofollow'] ? ' rel="nofollow"' : '';

        }

        if ( 'delayed' === $settings[ 'properties_type_animation' ] ) {

            $delayed = 'data-delay="' . $settings['properties_animation_delay' ][ 'size' ] . '"';

        }

        ?>

        <style>
            <?php foreach ( $settings['path_priority'] as $key => $val ): ?>
            #mdp-liner-<?php echo $this->get_id(); ?> svg path:nth-child(<?php echo esc_attr($val['path_number']); ?>){
                stroke: <?php echo esc_attr($val['path_style_color']); ?>;
                stroke-width: <?php echo esc_attr($val['path_style_line_width']['size']).esc_attr($val['path_style_line_width']['unit']); ?>;
                transform: translate( <?php echo esc_attr($val['path_style_offset_x']['size']).esc_attr($val['path_style_offset_x']['unit']); ?>,
                                    <?php echo esc_attr($val['path_style_offset_y']['size']).esc_attr($val['path_style_offset_y']['unit']); ?>)
                rotate(<?php echo esc_attr($val['path_style_rotate']['size']); ?>deg);
            }
            <?php endforeach; ?>
        </style>

        <?php if( 'yes' === $settings['properties_show_link']): ?>
        <a href="<?php echo esc_url($settings['properties_link']['url']); ?>" <?php echo  esc_attr($target) . esc_attr($nofollow); ?>>
            <?php endif; ?>

            <div id="mdp-liner-<?php echo esc_attr($idSection); ?>"
                 class="mdp-liner-box mdp-liner-init"
                 data-start="<?php esc_attr_e( $settings[ 'properties_start_animation' ] ); ?>"
                 data-url="<?php esc_attr_e( $settings[ 'animation_content__image' ][ 'url' ] ); ?>"
                 data-type="<?php esc_html_e( $settings[ 'properties_type_animation' ] ) ?>"
                 data-duration="<?php esc_html_e($settings['properties_animation_duration' ][ 'size' ] ) ?>"
                 data-dash="<?php esc_html_e( $settings['properties_dash_gap']['size'] ) ?>"
                 data-reverse="<?php esc_attr_e( $settings[ 'properties_reverse_stack' ] === 'yes' ? 'true' : 'false' ); ?>"
                 data-anim="<?php esc_attr_e( $settings[ 'properties_animation_timing' ] ); ?>"
                 data-path="<?php esc_attr_e( $settings[ 'properties_path_timing' ] ); ?>"
                 data-loop="<?php esc_attr_e( 'yes' === $settings[ 'loop' ] ? 'true' : 'false' ); ?>"
                <?php esc_html_e( $delayed ); ?>
            >
            </div>

            <?php if( 'yes' === $settings['properties_show_link']): ?>
        </a>
    <?php endif;

        if( 'delayed' === $settings[ 'properties_type_animation' ] && $settings[ 'properties_animation_delay' ][ 'size' ] > $settings[ 'properties_animation_duration' ][ 'size' ] ): ?>
            <p class="mdp-warning"><?php echo esc_html__( "Warning: Delay must be shorter than duration.", "liner-elementor" )?></p>
        <?php endif; ?>
        <!--suppress JSUnresolvedFunction -->
        <script>
            <?php if ( ! is_admin() ) { ?>
            window.addEventListener( 'elementor/frontend/init', function () {
                window.Liner( '<?php esc_attr_e($idSection); ?>' );
            } );
            <?php } else { ?>
            jQuery( document ).ready( function () { window.Liner( '<?php esc_attr_e($idSection); ?>' ) } );
            <?php } ?>
        </script>
        <?php
    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @since 1.0.0
     * @access public
     **/
    public function get_custom_help_url() {
        return 'https://docs.merkulov.design/tag/liner/';
    }

}
