<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Logger for Elementor
 * Logger creates a modern and stylish Elementor widget to display Changelog.
 * Exclusively on https://1.envato.market/logger-elementor
 *
 * @encoding        UTF-8
 * @version         1.0.5
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;
use Merkulove\Selection\Unity\ElementorControls;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Text_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Icons_Manager;
use Elementor\Repeater;
use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;

/** @noinspection PhpUnused */
/**
 * Logger - Custom Elementor Widget.
 **/
class logger_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style( 'mdp-selection-elementor-admin', UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css', [], UnityPlugin::get_version() );
        wp_register_style( 'mdp-logger', UnityPlugin::get_url() . 'css/logger' . UnityPlugin::get_suffix() . '.css', [], UnityPlugin::get_version() );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-logger-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Changelog', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-logger-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Changelog', 'List', 'Log', 'Changelog', 'Notes', 'Release' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-logger', 'mdp-selection-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [];

    }

    /**
     * Add the widget controls.
     *
     * @since 1.0.0
     * @access protected
     *
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        if ( true ) {

            /** Title Section. */
            if ( true ) {

                $this->start_controls_section( 'section_title', ['label' => esc_html__( 'Title', 'selection' ), 'tab' => Controls_Manager::TAB_CONTENT] );

                # Title
                $this->add_control(
                    'title',
                    [
                        'label'       => esc_html__( 'Title', 'selection' ),
                        'type'        => Controls_Manager::TEXTAREA,
                        'dynamic'     => ['active' => true],
                        'default'     => esc_html__( '1.0.0 RC 1 ', 'selection' ) . date('(d F Y)'),
                        'placeholder' => esc_html__( 'Enter your title', 'selection' ),
                        'label_block' => true
                    ]
                );

                # Link
                $this->add_control(
                    'title_link',
                    [
                        'label'     => esc_html__( 'Link', 'selection' ),
                        'type'      => Controls_Manager::URL,
                        'dynamic'   => ['active' => true],
                        'default'   => ['url' => ''],
                        'separator' => 'before'
                    ]
                );

                # HTML Tag
                $this->add_control(
                    'title_tag',
                    [
                        'label'     => esc_html__( 'HTML Tag', 'selection' ),
                        'type'      => Controls_Manager::SELECT,
                        'options'   => [
                            'h1'    => 'H1',
                            'h2'    => 'H2',
                            'h3'    => 'H3',
                            'h4'    => 'H4',
                            'h5'    => 'H5',
                            'h6'    => 'H6',
                            'div'   => 'div',
                            'span'  => 'span',
                            'p'     => 'p',
                        ],
                        'default'   => 'h3',
                    ]
                );

                # Alignment
                $this->add_responsive_control(
                    'title_align',
                    [
                        'label' => esc_html__( 'Alignment', 'selection' ),
                        'type' => Controls_Manager::CHOOSE,
                        'options' => [
                            'left' => [
                                'title' => esc_html__( 'Left', 'selection' ),
                                'icon' => 'eicon-text-align-left',
                            ],
                            'center' => [
                                'title' => esc_html__( 'Center', 'selection' ),
                                'icon' => 'eicon-text-align-center',
                            ],
                            'right' => [
                                'title' => esc_html__( 'Right', 'selection' ),
                                'icon' => 'eicon-text-align-right',
                            ],
                            'justify' => [
                                'title' => esc_html__( 'Justified', 'selection' ),
                                'icon' => 'eicon-text-align-justify',
                            ],
                        ],
                        'default' => '',
                        'selectors' => [
                            '{{WRAPPER}} .mdp-logger-elementor-title' => 'text-align: {{VALUE}};',
                        ],
                    ]
                );

                $this->end_controls_section();

            }

            /** Changelog Section. */
            if ( true ) {

                $this->start_controls_section( 'section_changelog', ['label' => esc_html__( 'Changelog', 'selection' ), 'tab' => Controls_Manager::TAB_CONTENT] );

                # Changelog Repeater
                $changelog = new Repeater();

                # Type
                $changelog->add_control(
                    'changelog_type',
                    [
                        'label'     => esc_html__( 'Type', 'selection' ),
                        'type'      => Controls_Manager::SELECT,
                        'dynamic'   => ['active' => false],
                        'options'   => [
                            'Added'         => esc_html__( 'Added', 'selection' ),
                            'Fixed'         => esc_html__( 'Fixed', 'selection' ),
                            'Changed'       => esc_html__( 'Changed', 'selection' ),
                            'Removed'       => esc_html__( 'Removed', 'selection' ),
                            'Updated'       => esc_html__( 'Updated', 'selection' ),
                            'Deprecated'    => esc_html__( 'Deprecated', 'selection' ),
                        ],
                        'default' => 'Added',
                        'label_block' => false,
                    ]
                );

                # Description
                $changelog->add_control(
                    'changelog_desc',
                    [
                        'label'         => esc_html__( 'Description', 'armada-core' ),
                        'type'          => Controls_Manager::TEXTAREA,
                        'dynamic'       => ['active' => false],
                        'label_block'   => true,
                        'default'       => '',
                        'placeholder'   => esc_html__( 'Enter a description to change', 'selection' ),
                    ]
                );

                # Changelog
                $this->add_control(
                    'changelog',
                    [
                        'label'         => '',
                        'type'          => Controls_Manager::REPEATER,
                        'fields'        => $changelog->get_controls(),
                        'default'       => [
                            [
                                'changelog_type' => 'Added',
                                'changelog_desc' => esc_html__( 'Customizable step sizes, frames per second etc.', 'selection' ),
                            ],[
                                'changelog_type' => 'Fixed',
                                'changelog_desc' => esc_html__( 'Full Code Refactoring.', 'selection' ),
                            ],[
                                'changelog_type' => 'Changed',
                                'changelog_desc' => esc_html__( 'Optimized CSS and JS files.', 'selection' ),
                            ]
                        ],
                        'title_field'   => '{{ changelog_type }}: {{ changelog_desc }}'
                    ]
                );

                $this->end_controls_section();

            }

        } // END Content Tab.

        /** Style Tab */
        if ( true ) {

            /** Title Style Section. */
            if ( true ) {

                $this->start_controls_section( 'section_title_style', ['label' => __( 'Title', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE] );

                # Color
                /** @noinspection PhpUndefinedClassInspection */
                $this->add_control(
                    'title_color',
                    [
                        'label'     => esc_html__( 'Color', 'selection' ),
                        'type'      => Controls_Manager::COLOR,
                        'scheme'    => [
                            'type'  => Color::get_type(),
                            'value' => Color::COLOR_3,
                        ],
                        'selectors' => [
                            /** Stronger selector to avoid section style from overwriting. */
                            "
                                {{WRAPPER}} .mdp-logger-elementor-box > .mdp-logger-elementor-title,
                                {{WRAPPER}} .mdp-logger-elementor-box > .mdp-logger-elementor-title a
                                " => 'color: {{VALUE}};',
                        ]
                    ]
                );

                # Typography
                $this->add_group_control(
                    Group_Control_Typography::get_type(),
                    [
                        'name' => 'title_typography',
                        'scheme' => Typography::TYPOGRAPHY_1,
                        'selector' => "
                                {{WRAPPER}} .mdp-logger-elementor-title > *,
                                {{WRAPPER}} .mdp-logger-elementor-title > * > a
                            ",
                    ]
                );

                # Text Shadow
                $this->add_group_control(
                    Group_Control_Text_Shadow::get_type(),
                    [
                        'name' => 'title_text_shadow',
                        'selector' => "
                                    {{WRAPPER}} .mdp-logger-elementor-title > *,
                                    {{WRAPPER}} .mdp-logger-elementor-title > * > a
                            ",
                    ]
                );

                # Blend Mode
                $this->add_control(
                    'title_blend_mode',
                    [
                        'label' => esc_html__( 'Blend Mode', 'selection' ),
                        'type'  => Controls_Manager::SELECT,
                        'options' => [
                            ''              => esc_html__( 'Normal', 'selection' ),
                            'multiply'      => esc_html__( 'Multiply', 'selection' ),
                            'screen'        => esc_html__( 'Screen', 'selection' ),
                            'overlay'       => esc_html__( 'Overlay', 'selection' ),
                            'darken'        => esc_html__( 'Darken', 'selection' ),
                            'lighten'       => esc_html__( 'Lighten', 'selection' ),
                            'color-dodge'   => esc_html__( 'Color Dodge', 'selection' ),
                            'saturation'    => esc_html__( 'Saturation', 'selection' ),
                            'color'         => esc_html__( 'Color', 'selection' ),
                            'difference'    => esc_html__( 'Difference', 'selection' ),
                            'exclusion'     => esc_html__( 'Exclusion', 'selection' ),
                            'hue'           => esc_html__( 'Hue', 'selection' ),
                            'luminosity'    => esc_html__( 'Luminosity', 'selection' ),
                        ],
                        'selectors' => [
                            "{{WRAPPER}} .mdp-logger-elementor-title > *" => 'mix-blend-mode: {{VALUE}}',
                            "{{WRAPPER}} .mdp-logger-elementor-title > * > a" => 'mix-blend-mode: {{VALUE}}',
                        ],
                        'separator' => 'none',
                    ]
                );

                # Margin
                $this->add_responsive_control(
                    'title_margin',
                    [
                        'label'         => esc_html__( 'Margin', 'selection' ),
                        'type'          => Controls_Manager::DIMENSIONS,
                        'default'       => [ 'top' => 0, 'right' => 0, 'bottom' => 0, 'left' => 0 ],
                        'size_units'    => [ 'px', 'em', '%' ],
                        'selectors'     => [
                            '{{WRAPPER}} .mdp-logger-elementor-title' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                        ]
                    ]
                );

                # Padding
                $this->add_responsive_control(
                    'title_padding',
                    [
                        'label'         => esc_html__( 'Padding', 'selection' ),
                        'type'          => Controls_Manager::DIMENSIONS,
                        'default'       => [ 'top' => 0, 'right' => 0, 'bottom' => 0, 'left' => 0 ],
                        'size_units'    => [ 'px', 'em', '%' ],
                        'selectors'     => [
                            '{{WRAPPER}} .mdp-logger-elementor-title' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                        ],
                        'separator' => 'after'
                    ]
                );

                # Background
                $this->add_group_control(
                    Group_Control_Background::get_type(),
                    [
                        'name'              => 'title_background',
                        'types'             => [ 'classic', 'gradient' ],
                        'selector'          => '{{WRAPPER}} .mdp-logger-elementor-title',
                    ]
                );

                # Border
                $this->add_group_control(
                    Group_Control_Border::get_type(),
                    [
                        'name' => 'title_border',
                        'selector' => '{{WRAPPER}} .mdp-logger-elementor-title',
                        'separator' => 'before',
                    ]
                );

                # Border Radius
                $this->add_control(
                    'title_radius',
                    [
                        'label' => __( 'Border Radius', 'selection' ),
                        'type' => Controls_Manager::DIMENSIONS,
                        'default'       => [ 'top' => 4, 'right' => 4, 'bottom' => 4, 'left' => 4 ],
                        'size_units' => [ 'px', '%' ],
                        'selectors' => [
                            '{{WRAPPER}} .mdp-logger-elementor-title' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                        ],
                        'separator' => 'after',
                    ]
                );

                $this->end_controls_section();

            } // END Title Style Section.

            /** Changelog Style Section. */
            if ( true ) {

                $this->start_controls_section( 'section_changelog_style', ['label' => __( 'Changelog', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE] );

                # Margin
                $this->add_responsive_control(
                    'changelog_margin',
                    [
                        'label'         => esc_html__( 'Margin', 'selection' ),
                        'type'          => Controls_Manager::DIMENSIONS,
                        'default'       => [ 'top' => 0, 'right' => 0, 'bottom' => 0, 'left' => 0 ],
                        'size_units'    => [ 'px', 'em', '%' ],
                        'selectors'     => [
                            '{{WRAPPER}} .mdp-logger-elementor-changelog' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                        ]
                    ]
                );

                # Padding
                $this->add_responsive_control(
                    'changelog_padding',
                    [
                        'label'         => esc_html__( 'Padding', 'selection' ),
                        'type'          => Controls_Manager::DIMENSIONS,
                        'default'       => [ 'top' => 0, 'right' => 0, 'bottom' => 0, 'left' => 0 ],
                        'size_units'    => [ 'px', 'em', '%' ],
                        'selectors'     => [
                            '{{WRAPPER}} .mdp-logger-elementor-changelog' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                        ],
                        'separator' => 'after'
                    ]
                );

                # Background
                $this->add_group_control(
                    Group_Control_Background::get_type(),
                    [
                        'name'              => 'changelog_background',
                        'types'             => [ 'classic', 'gradient' ],
                        'selector'          => '{{WRAPPER}} .mdp-logger-elementor-changelog',
                    ]
                );

                # Border
                $this->add_group_control(
                    Group_Control_Border::get_type(),
                    [
                        'name' => 'changelog_border',
                        'selector' => '{{WRAPPER}} .mdp-logger-elementor-changelog',
                        'separator' => 'before',
                    ]
                );

                # Border Radius
                $this->add_control(
                    'changelog_radius',
                    [
                        'label' => __( 'Border Radius', 'selection' ),
                        'type' => Controls_Manager::DIMENSIONS,
                        'default'       => [ 'top' => 4, 'right' => 4, 'bottom' => 4, 'left' => 4 ],
                        'size_units' => [ 'px', '%' ],
                        'selectors' => [
                            '{{WRAPPER}} .mdp-logger-elementor-changelog' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                        ],
                        'separator' => 'after',
                    ]
                );

                $this->end_controls_section();

            } // END Changelog Style Section.

            /** Changelog Item Style Section. */
            if ( true ) {

                $this->start_controls_section( 'section_changelog_item_style', ['label' => __( 'Changelog Item', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE] );

                # Margin
                $this->add_responsive_control(
                    'changelog_item_margin',
                    [
                        'label'         => esc_html__( 'Margin', 'selection' ),
                        'type'          => Controls_Manager::DIMENSIONS,
                        'default'       => [ 'top' => 0, 'right' => 0, 'bottom' => 0, 'left' => 0 ],
                        'size_units'    => [ 'px', 'em', '%' ],
                        'selectors'     => [
                            '{{WRAPPER}} .mdp-logger-elementor-changelog li' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                        ]
                    ]
                );

                # Padding
                $this->add_responsive_control(
                    'changelog_item_padding',
                    [
                        'label'         => esc_html__( 'Padding', 'selection' ),
                        'type'          => Controls_Manager::DIMENSIONS,
                        'default'       => [ 'top' => 0, 'right' => 0, 'bottom' => 0, 'left' => 0 ],
                        'size_units'    => [ 'px', 'em', '%' ],
                        'selectors'     => [
                            '{{WRAPPER}} .mdp-logger-elementor-changelog li' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                        ],
                        'separator' => 'after'
                    ]
                );

                # Background
                $this->add_group_control(
                    Group_Control_Background::get_type(),
                    [
                        'name'              => 'changelog_item_background',
                        'types'             => [ 'classic', 'gradient' ],
                        'selector'          => '{{WRAPPER}} .mdp-logger-elementor-changelog li',
                    ]
                );

                # Border
                $this->add_group_control(
                    Group_Control_Border::get_type(),
                    [
                        'name' => 'changelog_item_border',
                        'selector' => '{{WRAPPER}} .mdp-logger-elementor-changelog li',
                        'separator' => 'before',
                    ]
                );

                # Border Radius
                $this->add_control(
                    'changelog_item_radius',
                    [
                        'label' => __( 'Border Radius', 'selection' ),
                        'type' => Controls_Manager::DIMENSIONS,
                        'default'       => [ 'top' => 4, 'right' => 4, 'bottom' => 4, 'left' => 4 ],
                        'size_units' => [ 'px', '%' ],
                        'selectors' => [
                            '{{WRAPPER}} .mdp-logger-elementor-changelog li' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                        ],
                        'separator' => 'after',
                    ]
                );

                $this->end_controls_section();

            } // END Changelog Item Style Section.

            /** Changelog Type Style Section. */
            if ( true ) {

                $this->start_controls_section( 'section_changelog_type_style', ['label' => __( 'Changelog Type', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE] );

                # Type Size
                $this->add_responsive_control(
                    'changelog_type_size',
                    [
                        'label' => __( 'Type Size', 'elementor' ),
                        'type' => Controls_Manager::SLIDER,
                        'range' => [
                            'px' => [
                                'min' => 30,
                                'max' => 300,
                            ],
                        ],
                        'selectors' => [
                            '{{WRAPPER}} .mdp-logger-elementor-changelog .mdp-logger-elementor-changelog-type' => 'width: {{SIZE}}{{UNIT}};',
                        ],
                    ]
                );

                # Typography
                $this->add_group_control(
                    Group_Control_Typography::get_type(),
                    [
                        'name' => 'changelog_type_typography',
                        'scheme' => Typography::TYPOGRAPHY_3,
                        'selector' => "{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type",
                    ]
                );

                # Text Shadow
                $this->add_group_control(
                    Group_Control_Text_Shadow::get_type(),
                    [
                        'name' => 'changelog_type_text_shadow',
                        'selector' => "{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type",
                    ]
                );

                # Blend Mode
                $this->add_control(
                    'changelog_type_blend_mode',
                    [
                        'label' => esc_html__( 'Blend Mode', 'selection' ),
                        'type'  => Controls_Manager::SELECT,
                        'options' => [
                            ''              => esc_html__( 'Normal', 'selection' ),
                            'multiply'      => esc_html__( 'Multiply', 'selection' ),
                            'screen'        => esc_html__( 'Screen', 'selection' ),
                            'overlay'       => esc_html__( 'Overlay', 'selection' ),
                            'darken'        => esc_html__( 'Darken', 'selection' ),
                            'lighten'       => esc_html__( 'Lighten', 'selection' ),
                            'saturation'    => esc_html__( 'Saturation', 'selection' ),
                            'color-dodge'   => esc_html__( 'Color Dodge', 'selection' ),
                            'color'         => esc_html__( 'Color', 'selection' ),
                            'exclusion'     => esc_html__( 'Exclusion', 'selection' ),
                            'difference'    => esc_html__( 'Difference', 'selection' ),
                            'hue'           => esc_html__( 'Hue', 'selection' ),
                            'luminosity'    => esc_html__( 'Luminosity', 'selection' ),
                        ],
                        'selectors' => [
                            ".mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type" => 'mix-blend-mode: {{VALUE}}'
                        ],
                        'separator' => 'after'
                    ]
                );

                # Margin
                $this->add_responsive_control(
                    'changelog_type_margin',
                    [
                        'label'         => esc_html__( 'Margin', 'selection' ),
                        'type'          => Controls_Manager::DIMENSIONS,
                        'default'       => [ 'top' => 0, 'right' => 0, 'bottom' => 0, 'left' => 0 ],
                        'size_units'    => [ 'px', 'em', '%' ],
                        'selectors'     => [
                            '{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                        ]
                    ]
                );

                # Padding
                $this->add_responsive_control(
                    'changelog_type_padding',
                    [
                        'label'         => esc_html__( 'Padding', 'selection' ),
                        'type'          => Controls_Manager::DIMENSIONS,
                        'default'       => [ 'top' => 0, 'right' => 0, 'bottom' => 0, 'left' => 0 ],
                        'size_units'    => [ 'px', 'em', '%' ],
                        'selectors'     => [
                            '{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                        ],
                    ]
                );

                # Border Radius
                $this->add_control(
                    'changelog_type_radius',
                    [
                        'label' => __( 'Border Radius', 'selection' ),
                        'type' => Controls_Manager::DIMENSIONS,
                        'default'       => [ 'top' => 4, 'right' => 4, 'bottom' => 4, 'left' => 4 ],
                        'size_units' => [ 'px', '%' ],
                        'selectors' => [
                            '{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                        ],
                        'separator' => 'before',
                    ]
                );

                $this->end_controls_section();

            } // END Changelog Type Style Section.

            /** Changelog Description Style Section. */
            if ( true ) {

                $this->start_controls_section( 'section_changelog_desc_style', ['label' => __( 'Changelog Description', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE] );

                # Color
                /** @noinspection PhpUndefinedClassInspection */
                $this->add_control(
                    'changelog_desc_color',
                    [
                        'label'     => esc_html__( 'Color', 'selection' ),
                        'separator' => 'before',
                        'type'      => Controls_Manager::COLOR,
                        'scheme'    => [
                            'type'  => Color::get_type(),
                            'value' => Color::COLOR_3,
                        ],
                        'selectors' => [
                            /** Stronger selector to avoid section style from overwriting. */
                            "{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-desc" => 'color: {{VALUE}};'
                        ]
                    ]
                );

                # Typography
                $this->add_group_control(
                    Group_Control_Typography::get_type(),
                    [
                        'name' => 'changelog_desc_typography',
                        'scheme' => Typography::TYPOGRAPHY_3,
                        'selector' => "{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-desc",
                    ]
                );

                # Text Shadow
                $this->add_group_control(
                    Group_Control_Text_Shadow::get_type(),
                    [
                        'name' => 'changelog_desc_text_shadow',
                        'selector' => "{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-desc",
                    ]
                );

                # Blend Mode
                $this->add_control(
                    'changelog_desc_blend_mode',
                    [
                        'label' => esc_html__( 'Blend Mode', 'selection' ),
                        'type'  => Controls_Manager::SELECT,
                        'options' => [
                            ''              => esc_html__( 'Normal', 'selection' ),
                            'multiply'      => esc_html__( 'Multiply', 'selection' ),
                            'screen'        => esc_html__( 'Screen', 'selection' ),
                            'overlay'       => esc_html__( 'Overlay', 'selection' ),
                            'darken'        => esc_html__( 'Darken', 'selection' ),
                            'lighten'       => esc_html__( 'Lighten', 'selection' ),
                            'saturation'    => esc_html__( 'Saturation', 'selection' ),
                            'color-dodge'   => esc_html__( 'Color Dodge', 'selection' ),
                            'color'         => esc_html__( 'Color', 'selection' ),
                            'exclusion'     => esc_html__( 'Exclusion', 'selection' ),
                            'difference'    => esc_html__( 'Difference', 'selection' ),
                            'luminosity'    => esc_html__( 'Luminosity', 'selection' ),
                            'hue'           => esc_html__( 'Hue', 'selection' ),
                        ],
                        'selectors' => [
                            ".mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-desc" => 'mix-blend-mode: {{VALUE}}'
                        ],
                        'separator' => 'after'
                    ]
                );

                # Margin
                $this->add_responsive_control(
                    'changelog_desc_margin',
                    [
                        'label'         => esc_html__( 'Margin', 'selection' ),
                        'type'          => Controls_Manager::DIMENSIONS,
                        'default'       => [ 'top' => 0, 'right' => 0, 'bottom' => 0, 'left' => 0 ],
                        'size_units'    => [ 'px', 'em', '%' ],
                        'selectors'     => [
                            '{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-desc' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                        ]
                    ]
                );

                # Padding
                $this->add_responsive_control(
                    'changelog_desc_padding',
                    [
                        'label'         => esc_html__( 'Padding', 'selection' ),
                        'type'          => Controls_Manager::DIMENSIONS,
                        'default'       => [ 'top' => 0, 'right' => 0, 'bottom' => 0, 'left' => 0 ],
                        'size_units'    => [ 'px', 'em', '%' ],
                        'selectors'     => [
                            '{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-desc' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                        ],
                        'separator' => 'after'
                    ]
                );

                $this->end_controls_section();

            } // END Changelog Description Style Section.

            /** Added Type Style Section. */
            if ( true ) {

                $this->start_controls_section( 'section_changelog_added_style', ['label' => __( 'Added Type', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE] );

                # Icon
                $this->add_control(
                    'changelog_added_icon',
                    [
                        'label' => __( 'Icon', 'selection' ),
                        'type' => Controls_Manager::ICONS,
                        'fa4compatibility' => 'icon',
                        'default' => [
                            'value' => 'fas fa-star',
                            'library' => 'fa-solid',
                        ],
                    ]
                );

                # Icon Position
                $this->add_control(
                    'changelog_added_position',
                    [
                        'label' => esc_html__( 'Icon Position', 'selection' ),
                        'type' => Controls_Manager::CHOOSE,
                        'default' => 'left',
                        'options' => [
                            'left' => [
                                'title' => esc_html__( 'Left', 'selection' ),
                                'icon' => 'eicon-h-align-left',
                            ],
                            'top' => [
                                'title' => esc_html__( 'Top', 'selection' ),
                                'icon' => 'eicon-v-align-top',
                            ],
                            'right' => [
                                'title' => esc_html__( 'Right', 'selection' ),
                                'icon' => 'eicon-h-align-right',
                            ],
                        ],
                        'prefix_class' => 'mdp-logger-elementor-added-icon-position-',
                        'toggle' => false,
                        'conditions' => [
                            'relation' => 'or',
                            'terms' => [
                                [
                                    'name' => 'changelog_added_icon[value]',
                                    'operator' => '!=',
                                    'value' => '',
                                ],
                            ],
                        ]
                    ]
                );

                # Text Color
                /** @noinspection PhpUndefinedClassInspection */
                $this->add_control(
                    'changelog_added_color',
                    [
                        'label'     => esc_html__( 'Text Color', 'selection' ),
                        'separator' => 'before',
                        'type'      => Controls_Manager::COLOR,
                        'scheme'    => [
                            'type'  => Color::get_type(),
                            'value' => Color::COLOR_1,
                        ],
                        'selectors' => [
                            /** Stronger selector to avoid section style from overwriting. */
                            "{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type-added" => 'color: {{VALUE}};'
                        ]
                    ]
                );

                # Background
                $this->add_group_control(
                    Group_Control_Background::get_type(),
                    [
                        'name'              => 'changelog_added_background',
                        'types'             => [ 'classic', 'gradient' ],
                        'separator'         => 'before',
                        'selector'          => '{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type-added',
                    ]
                );

                # Border
                $this->add_group_control(
                    Group_Control_Border::get_type(),
                    [
                        'name' => 'changelog_added_border',
                        'selector' => '{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type-added',
                        'separator' => 'before',
                    ]
                );

                $this->end_controls_section();

            } // END Added Type Style Section.

            /** Fixed Type Style Section. */
            if ( true ) {

                $this->start_controls_section( 'section_changelog_fixed_style', ['label' => __( 'Fixed Type', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE] );

                # Icon
                $this->add_control(
                    'changelog_fixed_icon',
                    [
                        'label' => __( 'Icon', 'selection' ),
                        'type' => Controls_Manager::ICONS,
                        'fa4compatibility' => 'icon',
                        'default' => [
                            'value' => 'fas fa-star',
                            'library' => 'fa-solid',
                        ],
                    ]
                );

                # Icon Position
                $this->add_control(
                    'changelog_fixed_position',
                    [
                        'label' => esc_html__( 'Icon Position', 'selection' ),
                        'type' => Controls_Manager::CHOOSE,
                        'default' => 'left',
                        'options' => [
                            'left' => [
                                'title' => esc_html__( 'Left', 'selection' ),
                                'icon' => 'eicon-h-align-left',
                            ],
                            'top' => [
                                'title' => esc_html__( 'Top', 'selection' ),
                                'icon' => 'eicon-v-align-top',
                            ],
                            'right' => [
                                'title' => esc_html__( 'Right', 'selection' ),
                                'icon' => 'eicon-h-align-right',
                            ],
                        ],
                        'prefix_class' => 'mdp-logger-elementor-fixed-icon-position-',
                        'toggle' => false,
                        'conditions' => [
                            'relation' => 'or',
                            'terms' => [
                                [
                                    'name' => 'changelog_fixed_icon[value]',
                                    'operator' => '!=',
                                    'value' => '',
                                ],
                            ],
                        ]
                    ]
                );

                # Text Color
                /** @noinspection PhpUndefinedClassInspection */
                $this->add_control(
                    'changelog_fixed_color',
                    [
                        'label'     => esc_html__( 'Text Color', 'selection' ),
                        'separator' => 'before',
                        'type'      => Controls_Manager::COLOR,
                        'scheme'    => [
                            'type'  => Color::get_type(),
                            'value' => Color::COLOR_1,
                        ],
                        'selectors' => [
                            /** Stronger selector to avoid section style from overwriting. */
                            "{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type-fixed" => 'color: {{VALUE}};'
                        ]
                    ]
                );

                # Background
                $this->add_group_control(
                    Group_Control_Background::get_type(),
                    [
                        'name'              => 'changelog_fixed_background',
                        'types'             => [ 'classic', 'gradient' ],
                        'separator'         => 'before',
                        'selector'          => '{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type-fixed',
                    ]
                );

                # Border
                $this->add_group_control(
                    Group_Control_Border::get_type(),
                    [
                        'name' => 'changelog_fixed_border',
                        'selector' => '{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type-fixed',
                        'separator' => 'before',
                    ]
                );

                $this->end_controls_section();

            } // END Fixed Type Style Section.

            /** Changed Type Style Section. */
            if ( true ) {

                $this->start_controls_section( 'section_changelog_changed_style', ['label' => __( 'Changed Type', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE] );

                # Icon
                $this->add_control(
                    'changelog_changed_icon',
                    [
                        'label' => __( 'Icon', 'selection' ),
                        'type' => Controls_Manager::ICONS,
                        'fa4compatibility' => 'icon',
                        'default' => [
                            'value' => 'fas fa-star',
                            'library' => 'fa-solid',
                        ],
                    ]
                );

                # Icon Position
                $this->add_control(
                    'changelog_changed_position',
                    [
                        'label' => esc_html__( 'Icon Position', 'selection' ),
                        'type' => Controls_Manager::CHOOSE,
                        'default' => 'left',
                        'options' => [
                            'left' => [
                                'title' => esc_html__( 'Left', 'selection' ),
                                'icon' => 'eicon-h-align-left',
                            ],
                            'top' => [
                                'title' => esc_html__( 'Top', 'selection' ),
                                'icon' => 'eicon-v-align-top',
                            ],
                            'right' => [
                                'title' => esc_html__( 'Right', 'selection' ),
                                'icon' => 'eicon-h-align-right',
                            ],
                        ],
                        'prefix_class' => 'mdp-logger-elementor-changed-icon-position-',
                        'toggle' => false,
                        'conditions' => [
                            'relation' => 'or',
                            'terms' => [
                                [
                                    'name' => 'changelog_changed_icon[value]',
                                    'operator' => '!=',
                                    'value' => '',
                                ],
                            ],
                        ]
                    ]
                );

                # Text Color
                /** @noinspection PhpUndefinedClassInspection */
                $this->add_control(
                    'changelog_changed_color',
                    [
                        'label'     => esc_html__( 'Text Color', 'selection' ),
                        'separator' => 'before',
                        'type'      => Controls_Manager::COLOR,
                        'scheme'    => [
                            'type'  => Color::get_type(),
                            'value' => Color::COLOR_1,
                        ],
                        'selectors' => [
                            /** Stronger selector to avoid section style from overwriting. */
                            "{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type-changed" => 'color: {{VALUE}};'
                        ]
                    ]
                );

                # Background
                $this->add_group_control(
                    Group_Control_Background::get_type(),
                    [
                        'name'              => 'changelog_changed_background',
                        'types'             => [ 'classic', 'gradient' ],
                        'separator'         => 'before',
                        'selector'          => '{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type-changed',
                    ]
                );

                # Border
                $this->add_group_control(
                    Group_Control_Border::get_type(),
                    [
                        'name' => 'changelog_changed_border',
                        'selector' => '{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type-changed',
                        'separator' => 'before',
                    ]
                );

                $this->end_controls_section();

            } // END Changed Type Style Section.

            /** Removed Type Style Section. */
            if ( true ) {

                $this->start_controls_section( 'section_changelog_removed_style', ['label' => __( 'Removed Type', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE] );

                # Icon
                $this->add_control(
                    'changelog_removed_icon',
                    [
                        'label' => __( 'Icon', 'selection' ),
                        'type' => Controls_Manager::ICONS,
                        'fa4compatibility' => 'icon',
                        'default' => [
                            'value' => 'fas fa-star',
                            'library' => 'fa-solid',
                        ],
                    ]
                );

                # Icon Position
                $this->add_control(
                    'changelog_removed_position',
                    [
                        'label' => esc_html__( 'Icon Position', 'selection' ),
                        'type' => Controls_Manager::CHOOSE,
                        'default' => 'left',
                        'options' => [
                            'left' => [
                                'title' => esc_html__( 'Left', 'selection' ),
                                'icon' => 'eicon-h-align-left',
                            ],
                            'top' => [
                                'title' => esc_html__( 'Top', 'selection' ),
                                'icon' => 'eicon-v-align-top',
                            ],
                            'right' => [
                                'title' => esc_html__( 'Right', 'selection' ),
                                'icon' => 'eicon-h-align-right',
                            ],
                        ],
                        'prefix_class' => 'mdp-logger-elementor-removed-icon-position-',
                        'toggle' => false,
                        'conditions' => [
                            'relation' => 'or',
                            'terms' => [
                                [
                                    'name' => 'changelog_removed_icon[value]',
                                    'operator' => '!=',
                                    'value' => '',
                                ],
                            ],
                        ]
                    ]
                );

                # Text Color
                /** @noinspection PhpUndefinedClassInspection */
                $this->add_control(
                    'changelog_removed_color',
                    [
                        'label'     => esc_html__( 'Text Color', 'selection' ),
                        'separator' => 'before',
                        'type'      => Controls_Manager::COLOR,
                        'scheme'    => [
                            'type'  => Color::get_type(),
                            'value' => Color::COLOR_1,
                        ],
                        'selectors' => [
                            /** Stronger selector to avoid section style from overwriting. */
                            "{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type-removed" => 'color: {{VALUE}};'
                        ]
                    ]
                );

                # Background
                $this->add_group_control(
                    Group_Control_Background::get_type(),
                    [
                        'name'              => 'changelog_removed_background',
                        'types'             => [ 'classic', 'gradient' ],
                        'separator'         => 'before',
                        'selector'          => '{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type-removed',
                    ]
                );

                # Border
                $this->add_group_control(
                    Group_Control_Border::get_type(),
                    [
                        'name' => 'changelog_removed_border',
                        'selector' => '{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type-removed',
                        'separator' => 'before',
                    ]
                );

                $this->end_controls_section();

            } // END Removed Type Style Section.

            /** Updated Type Style Section. */
            if ( true ) {

                $this->start_controls_section( 'section_changelog_updated_style', ['label' => __( 'Updated Type', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE] );

                # Icon
                $this->add_control(
                    'changelog_updated_icon',
                    [
                        'label' => __( 'Icon', 'selection' ),
                        'type' => Controls_Manager::ICONS,
                        'fa4compatibility' => 'icon',
                        'default' => [
                            'value' => 'fas fa-star',
                            'library' => 'fa-solid',
                        ],
                    ]
                );

                # Icon Position
                $this->add_control(
                    'changelog_updated_position',
                    [
                        'label' => esc_html__( 'Icon Position', 'selection' ),
                        'type' => Controls_Manager::CHOOSE,
                        'default' => 'left',
                        'options' => [
                            'left' => [
                                'title' => esc_html__( 'Left', 'selection' ),
                                'icon' => 'eicon-h-align-left',
                            ],
                            'top' => [
                                'title' => esc_html__( 'Top', 'selection' ),
                                'icon' => 'eicon-v-align-top',
                            ],
                            'right' => [
                                'title' => esc_html__( 'Right', 'selection' ),
                                'icon' => 'eicon-h-align-right',
                            ],
                        ],
                        'prefix_class' => 'mdp-logger-elementor-updated-icon-position-',
                        'toggle' => false,
                        'conditions' => [
                            'relation' => 'or',
                            'terms' => [
                                [
                                    'name' => 'changelog_updated_icon[value]',
                                    'operator' => '!=',
                                    'value' => '',
                                ],
                            ],
                        ]
                    ]
                );

                # Text Color
                /** @noinspection PhpUndefinedClassInspection */
                $this->add_control(
                    'changelog_updated_color',
                    [
                        'label'     => esc_html__( 'Text Color', 'selection' ),
                        'separator' => 'before',
                        'type'      => Controls_Manager::COLOR,
                        'scheme'    => [
                            'type'  => Color::get_type(),
                            'value' => Color::COLOR_1,
                        ],
                        'selectors' => [
                            /** Stronger selector to avoid section style from overwriting. */
                            "{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type-updated" => 'color: {{VALUE}};'
                        ]
                    ]
                );

                # Background
                $this->add_group_control(
                    Group_Control_Background::get_type(),
                    [
                        'name'              => 'changelog_updated_background',
                        'types'             => [ 'classic', 'gradient' ],
                        'separator'         => 'before',
                        'selector'          => '{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type-updated',
                    ]
                );

                # Border
                $this->add_group_control(
                    Group_Control_Border::get_type(),
                    [
                        'name' => 'changelog_updated_border',
                        'selector' => '{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type-updated',
                        'separator' => 'before',
                    ]
                );

                $this->end_controls_section();

            } // END Updated Type Style Section.

            /** Deprecated Type Style Section. */
            if ( true ) {

                $this->start_controls_section( 'section_changelog_deprecated_style', ['label' => __( 'Deprecated Type', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE] );

                # Icon
                $this->add_control(
                    'changelog_deprecated_icon',
                    [
                        'label' => __( 'Icon', 'selection' ),
                        'type' => Controls_Manager::ICONS,
                        'fa4compatibility' => 'icon',
                        'default' => [
                            'value' => 'fas fa-star',
                            'library' => 'fa-solid',
                        ],
                    ]
                );

                # Icon Position
                $this->add_control(
                    'changelog_deprecated_position',
                    [
                        'label' => esc_html__( 'Icon Position', 'selection' ),
                        'type' => Controls_Manager::CHOOSE,
                        'default' => 'left',
                        'options' => [
                            'left' => [
                                'title' => esc_html__( 'Left', 'selection' ),
                                'icon' => 'eicon-h-align-left',
                            ],
                            'top' => [
                                'title' => esc_html__( 'Top', 'selection' ),
                                'icon' => 'eicon-v-align-top',
                            ],
                            'right' => [
                                'title' => esc_html__( 'Right', 'selection' ),
                                'icon' => 'eicon-h-align-right',
                            ],
                        ],
                        'prefix_class' => 'mdp-logger-elementor-deprecated-icon-position-',
                        'toggle' => false,
                        'conditions' => [
                            'relation' => 'or',
                            'terms' => [
                                [
                                    'name' => 'changelog_deprecated_icon[value]',
                                    'operator' => '!=',
                                    'value' => '',
                                ],
                            ],
                        ]
                    ]
                );

                # Text Color
                /** @noinspection PhpUndefinedClassInspection */
                $this->add_control(
                    'changelog_deprecated_color',
                    [
                        'label'     => esc_html__( 'Text Color', 'selection' ),
                        'separator' => 'before',
                        'type'      => Controls_Manager::COLOR,
                        'scheme'    => [
                            'type'  => Color::get_type(),
                            'value' => Color::COLOR_1,
                        ],
                        'selectors' => [
                            /** Stronger selector to avoid section style from overwriting. */
                            "{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type-deprecated" => 'color: {{VALUE}};'
                        ]
                    ]
                );

                # Background
                $this->add_group_control(
                    Group_Control_Background::get_type(),
                    [
                        'name'              => 'changelog_deprecated_background',
                        'types'             => [ 'classic', 'gradient' ],
                        'separator'         => 'before',
                        'selector'          => '{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type-deprecated',
                    ]
                );

                # Border
                $this->add_group_control(
                    Group_Control_Border::get_type(),
                    [
                        'name' => 'changelog_deprecated_border',
                        'selector' => '{{WRAPPER}} .mdp-logger-elementor-changelog li .mdp-logger-elementor-changelog-type-deprecated',
                        'separator' => 'before',
                    ]
                );

                $this->end_controls_section();

            } // END Deprecated Type Style Section.

        } // END Style Tab.

    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @since 1.0.0
     * @access protected
     **/
    protected function render() {

        /** Get Widget Settings. */
        $settings = $this->get_settings_for_display();

        ?>
        <!-- Start Logger for Elementor WordPress Plugin -->
        <div class="mdp-logger-elementor-box">

        <?php if ( ! empty( $settings['title'] ) ) : ?>
            <div class="mdp-logger-elementor-title">

            <?php $tag = $settings['title_tag']; // HTML tag for title. ?>
            <?php $this->add_inline_editing_attributes( 'title', 'basic' ); ?>
            <<?php esc_attr_e( $tag ); ?> <?php echo $this->get_render_attribute_string( 'title' ); ?> >

            <?php if ( ! empty( $settings['title_link']['url'] ) ) : ?>
        <?php
        /** Prepare title link attributes. */
        $atts = '';
        if ( $settings['title_link']['is_external'] ) { $atts .= ' target="_blank" '; }
        if ( $settings['title_link']['nofollow'] ) { $atts .= ' rel="nofollow" '; }
        $atts = trim( $atts );
        ?>
            <a href="<?php echo esc_url( $settings['title_link']['url'] ); ?>" <?php esc_attr_e( $atts ); ?>>
        <?php endif; ?>

            <?php echo wp_kses_post( $settings['title'] ); ?>

            <?php if ( ! empty( $settings['title_link']['url'] ) ) : ?>
            </a>
        <?php endif; ?>

            </<?php esc_attr_e( $tag ); ?>>
            </div>
        <?php endif; ?>

        <?php if ( count( $settings['changelog'] ) ) : ?>
            <div class="mdp-logger-elementor-changelog">

                <?php $this->add_render_attribute( 'changelog', 'class', 'mdp-logger-elementor-changelog-ul' ); ?>
                <?php $this->add_render_attribute( 'changelog_item', 'class', 'mdp-logger-elementor-changelog-li' ); ?>

                <ul <?php echo $this->get_render_attribute_string( 'changelog' ); ?>>
                    <?php foreach ( $settings['changelog'] as $index => $record ) : ?>
                        <?php if ( ! empty( $record['changelog_desc'] ) ) : ?>

                            <?php $repeater_setting_key = $this->get_repeater_setting_key( 'changelog_desc', 'changelog', $index ); ?>
                            <?php $this->add_render_attribute( $repeater_setting_key, 'class', 'mdp-logger-elementor-changelog-desc' ); ?>
                            <?php $this->add_inline_editing_attributes( $repeater_setting_key ); ?>

                            <li <?php echo $this->get_render_attribute_string( 'changelog_item' ); ?>>
                                <?php $type = strtolower( $record['changelog_type'] ); ?>
                                <span class="mdp-logger-elementor-changelog-type mdp-logger-elementor-changelog-type-<?php esc_html_e( $type ); ?>">
                                        <?php if ( ! empty( $settings['changelog_' . $type . '_icon'] ) ) : ?>
                                            <?php Icons_Manager::render_icon( $settings['changelog_' . $type . '_icon'], [ 'aria-hidden' => 'true' ] ); ?>
                                        <?php endif; ?>
                                        <span><?php esc_html_e( $record['changelog_type'], 'selection' ); ?></span>
                                    </span>
                                <div <?php echo $this->get_render_attribute_string( $repeater_setting_key ); ?>><?php echo wp_kses_post( $record['changelog_desc'] ); ?></div>
                            </li>
                        <?php endif; ?>
                    <?php endforeach; ?>
                </ul>
            </div>
        <?php endif; ?>

        </div>
        <!-- End Logger for Elementor WordPress Plugin -->
        <?php
    }

    /**
     * Render Frontend output in the editor.
     *
     * Written as a Backbone JavaScript template and used to generate the live preview.
     *
     * @since 1.0.0
     * @access protected
     **/
    protected function content_template() {
        ?>
        <!-- Start Logger for Elementor WordPress Plugin -->
        <div class="mdp-logger-elementor-box">
            <div class="mdp-logger-elementor-title">
                <#
                var tag = settings.title_tag;
                view.addInlineEditingAttributes( 'title', 'basic' );
                view.addRenderAttribute( 'title' );
                #>
                <{{{ tag }}} {{{ view.getRenderAttributeString( 'title' ) }}}>
                    <#
                    var title_html = settings.title;
                    if ( '' !== settings.title_link.url ) {
                        var atts = '';
                        if ( '' !== settings.title_link.is_external ) { atts += ' target="_blank" '; }
                        if ( '' !== settings.title_link.nofollow ) { atts += ' rel="nofollow" '; }
                        atts = atts.trim();
                        title_html = '<a href="' + settings.title_link.url + '" ' + atts + '>' + settings.title + '</a>';
                    }
                    print( title_html );
                    #>
                </{{{ tag }}}>
            </div>

            <# if ( settings.changelog ) { #>
            <div class="mdp-logger-elementor-changelog">

                <# view.addRenderAttribute( 'changelog', 'class', 'mdp-logger-elementor-changelog-ul' ); #>
                <# view.addRenderAttribute( 'changelog_item', 'class', 'mdp-logger-elementor-changelog-li' ); #>

                <ul {{{ view.getRenderAttributeString( 'changelog' ) }}}>
                <#
                _.each( settings.changelog, function( record, index ) {
                var changelogDescKey = view.getRepeaterSettingKey( 'changelog_desc', 'changelog', index );
                view.addRenderAttribute( changelogDescKey, 'class', 'mdp-logger-elementor-changelog-desc' );
                // view.addInlineEditingAttributes( changelogDescKey );
                #>
                <li {{{ view.getRenderAttributeString( 'changelog_item' ) }}}>
                <# var type = record.changelog_type.toLowerCase(); #>
                <span class="mdp-logger-elementor-changelog-type mdp-logger-elementor-changelog-type-{{{ type }}}">
                                            <#
                                                if ( settings['changelog_' + type + '_icon'] ) {
                                                    var iconHTML = elementor.helpers.renderIcon( view, settings['changelog_' + type + '_icon'], { 'aria-hidden': true }, 'i' , 'object' );
                                                }
                                            #>
                                            {{{ iconHTML.value }}}
                                            <span>{{{ record.changelog_type }}}</span>
                                        </span>
                <div {{{ view.getRenderAttributeString( changelogDescKey ) }}} >{{{ record.changelog_desc }}}</div>
                </li>
                <# } );
                #>
                </ul>
            </div>
            <# } #>

        </div>
        <!-- End Logger for Elementor WordPress Plugin -->
        <?php
    }

    /**
     * Return link for documentation.
     *
     * Used to add stuff after widget.
     *
     * @since 1.0.0
     * @access public
     **/
    public function get_custom_help_url() {
        return 'https://docs.merkulov.design/category/logger-elementor/';
    }

}
