<?php
/**
 * Selection
 * Carefully selected Elementor addons bundle, for building most awesome websites
 * Exclusively on https://1.envato.market/selection
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

use Exception;
use Merkulove\Selection\Unity\Plugin;
use Merkulove\Selection\Unity\Settings;
use Elementor\Core\Common\Modules\Ajax\Module as Ajax;
use Merkulove\Selection\Unity\TabActivation;
use WP_REST_Server;

final class TemplatesLibrary {

	/**
	 * The one true TemplatesLibrary.
	 *
	 * @access private
	 * @var TemplatesLibrary
	 **/
	private static $instance;

	/**
     * Transients expiration time
	 * @var int
	 */
//    public static $transient_lifetime = DAY_IN_SECONDS;
    public static $transient_lifetime = 60;

	/**
     * Templates library host
	 * @var string
	 */
    public static $host = 'https://myselection.io';

	/**
     * Templates Library actions
	 * @return void
	 */
	public function setup() {

		/** Print views and tab variables on footer */
		add_action( 'elementor/editor/footer', [$this, 'script_variables'] );

		/** Enqueue editor js for elementor */
		add_action( 'elementor/editor/before_enqueue_scripts', [$this, 'editor_scripts'], 1);

		/** Enqueue editor css */
		add_action( 'elementor/editor/after_enqueue_styles', [$this, 'editor_styles'] );

		/** Enqueue modal's preview css */
		add_action( 'elementor/preview/enqueue_styles', [$this, 'preview_styles'] );

        /** Register template import action */
		add_action( 'elementor/ajax/register_actions', [ $this, 'register_ajax_actions' ] );

        /** Rest api route for get layouts list */
		add_action( 'rest_api_init', function () {
			register_rest_route( untrailingslashit('selection/v1'), '/templates/(?P<action>\w+)/', array(
				'methods' => WP_REST_Server::ALLMETHODS,
				'callback' => [ $this, 'rest_callback_library' ],
				'permission_callback' => '__return_true',
			));
		});

	}

	/**
     * Register ajax action
	 * @param Ajax $ajax
	 *
	 * @return void
	 */
	public function register_ajax_actions( Ajax $ajax ){

		$ajax->register_ajax_action( 'get_selection_template_data', function( $data ) {

            // Allow on if user have editor rights
			if ( ! current_user_can( 'edit_posts' ) ) {
				throw new Exception( 'Access Denied' );
			}

			// Allow on if post exist
			if ( ! empty( $data['editor_post_id'] ) ) {
				$editor_post_id = absint( $data['editor_post_id'] );

				if ( ! get_post( $editor_post_id ) ) {
					throw new Exception( esc_html__( 'Post not found', 'selection' ) );
				}

				\Elementor\Plugin::instance()->db->switch_to_post( $editor_post_id );
			}

			// Allow on if template is provided
			if ( empty( $data['template_id'] ) ) {
				throw new Exception( esc_html__( 'Template id missing', 'selection' ) );
			}

			return $this->get_template_data( $data );

		} );

	}

	/**
     * Get template data
     *
	 * @param array $args
	 *
	 * @return mixed
	 * @throws Exception
	 */
	private function get_template_data( array $args ) {

		$source = new TemplateLibrarySource();
		return $source->get_data( $args );

	}

	/**
     * Rest call callback for templates library list for get_layouts_list
	 * @return mixed
	 */
	public function rest_callback_library( $request ){

        // Get request parameters
        $params = $request->get_params();
		$library = json_decode( $this->get_transient_list() );

        if ( isset( $params[ 'categories' ] ) ) {

            /** Prepare parameters for filtering of the api response */
	        $category = $params[ 'categories' ];
	        $tab = $params[ 'tabs' ];

	        /** Filtered list of layouts from transient or api */
	        return $this->filter_library( $library, $category, $tab );

        } else {

            /** Request list of layouts from the api */
	        return $library;

        }

	}

	/**
     * Get layouts list object from api
	 * @return string
	 */
    private function api_list() {

        // Make a request
	    $response = wp_remote_get(
		    self::$host . '/wp-json/selection/v1/library/section',
		    [
			    'timeout'     => 30,
			    'headers'     => [
				    'Content-Type' => 'application/json',
			    ],
			    'sslverify'  => Settings::get_instance()->options[ 'check_ssl' ] === 'on'
		    ]
	    );

        // Get request body
	    $body = wp_remote_retrieve_body( $response );

	    // Store body to transient
        $this->set_api_transient( $body, 'mdp-selection-library' );

	    return $body;

    }

	/**
	 * Store response to transient option if it is valid json
	 *
	 * @param $body
	 * @param $key
	 *
	 * @return void
	 */
    private function set_api_transient( $body, $key ) {

	    if ( json_decode( $body ) !== null ) {

		    set_transient( $key, $body, self::$transient_lifetime );

	    }

    }

	/**
     * Get template data from api
	 * @param $template_id
	 *
	 * @return string
	 */
	public function api_template( $template_id ) {

		$query = array_merge([
			'template_id' => $template_id,
		], [ 'valid' ]);

		$response = wp_remote_get(
            self::$host . '/wp-json/selection/v1/library/import' . '?' . http_build_query($query),
			[
				'timeout'     => 30,
				'sslverify'  => Settings::get_instance()->options[ 'check_ssl' ] === 'on'
			]
		);

		// Get request body
        $body = wp_remote_retrieve_body( $response );

		// Store body to transient
        $this->set_api_transient( $body, 'mdp-selection-library-template-' . $template_id );

		return $body;

	}

	/**
     * Get layouts object from transient of api if transient expired
	 * @return mixed
	 */
    private function get_transient_list() {

	    $transient = get_transient( 'mdp-selection-library' );

        return $transient === false ? $this->api_list() : $transient;

    }

	/**
     * Get template object from transient of api if transient expired
	 * @param $template_id
	 *
	 * @return mixed|string
	 */
    public function get_transient_template( $template_id ) {

	    if ( empty( $template_id ) ) {
		    return '';
	    }

	    $transient = get_transient( 'mdp-selection-library-template-' . $template_id );

	    return $transient === false ? $this->api_template( $template_id ) : $transient;

    }

	/**
	 * Filter library templates by category
	 *
	 * @param $library
	 * @param $category
	 * @param string $tab
	 *
	 * @return mixed
	 */
    private function filter_library( $library, $category, $tab ) {

        // Don't filter anything if All categories selected
        if ( $category === '' ) { return $library; }

        // Prepare variables
	    $layouts_filtered = (object)array();
	    $filtered = 0;

        // Filter templates
	    foreach ( $library->layouts as $template ) {

            $categories = $template->categories;
            $tabs = $template->tabs;

		    if ( array_key_exists( $category, $categories ) && array_key_exists( $tab, $tabs ) ) {

			    $layouts_filtered->{ $template->get_searchable_unique_name } = $template;
			    $filtered++;

		    }

	    }

        // Update changed library object variables
	    $library->layouts = $layouts_filtered;
	    $library->layout_count->filtered = $filtered;

	    return $library;

    }

	/**
     * Enqueue scripts in Elementor editor
	 * @return void
	 */
	public function editor_scripts() {

		wp_enqueue_script(
            'mdp-selection-elementor-library',
            Plugin::get_url() . 'js/templates-library/templates-library' . Plugin::get_suffix() . '.js',
            [ 'jquery', 'wp-element' ],
            Plugin::get_version(), true
        );

	}

	/**
     * Enqueue styles in Elementor editor
	 * @return void
	 */
	public function editor_styles() {

		wp_enqueue_style(
            'mdp-selection-elementor-library',
            Plugin::get_url() . 'css/templates-library/templates-library' . Plugin::get_suffix() . '.css',
            [],
            Plugin::get_version()
        );

	}

	/**
     * Enqueue styles in for Elementor preview
	 * @return void
	 */
	public function preview_styles() {

		wp_enqueue_style(
            'mdp-selection-elementor-preview',
            Plugin::get_url() . 'css/templates-library/preview' . Plugin::get_suffix() . '.css',
            [],
            Plugin::get_version()
        );

	}

	/**
     * Templates Library variables
	 * @return void
	 */
	public function script_variables() {
        ?><script type="text/javascript">

            window.mdpSelectionLibrary = {
                "defaultTab": "section",
                "nonce": "<?php echo wp_create_nonce( 'wp_rest' ); ?>",
                "pluginHome": "<?php echo Plugin::get_url(); ?>",
                "apiUrl": "<?php echo get_rest_url( null, 'selection/v1/templates/layout_list' ); ?>",
                "licenseStatus": <?php echo TabActivation::get_instance()->is_activated() ? "true" : "false"; ?>,
                "links": {
                    "activateLicense": "?page=mdp_selection_settings&tab=activation"
                }
            };

		</script><?php
	}

	/**
	 * Main TemplatesLibrary Instance.
	 * Insures that only one instance of TemplatesLibrary exists in memory at any one time.
	 *
	 * @static
	 * @since 1.0.0
	 * @access public
	 *
	 * @return TemplatesLibrary
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
